/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.antgraphbuilder;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.StringTokenizer;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.support.AntScriptUtils;
import org.apache.tools.ant.module.api.support.TargetLister;
import org.apache.tools.ant.module.api.support.TargetLister.Target;
import org.gpdv.services.GraphBuilder;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.AbstractGraphFactory;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphBuilder.class)

public class AntGraphBuilderImpl implements GraphBuilder{

    private final String ANT = "text/x-ant+xml";

//    private final String NAME = "ANTGRAPHBUILDER";
//    private final String DISPLAYNAME = "Graph Builder for Ant Dependencies";
//    private final String DESCRIPTION = "Graph Builder accepting NetBeans Projects or FileObjects as input to build an AbstractGraph";

    private final String BUILDFILE = "build.xml";

    private final String ASTERISK = "*";
    private final String ANTDESCRIPTION = "description";
    private final String ANTIF = "if";
    private final String UNLESS = "unless";
    private final String DEPENDS = "depends";
    private final String RESOLVED = "Resolved";
    private final String TASK = "Task ";
    private final String TO = " -> ";

    private static ResourceBundle bundle = NbBundle.getBundle(AntGraphBuilderImpl.class);



    public Node getRepresentation() {
        Node n = new org.openide.nodes.AbstractNode(Children.LEAF);
        n.setName(bundle.getString("NAME"));
        n.setDisplayName(bundle.getString("DISPLAYNAME"));
        n.setShortDescription(bundle.getString("DESCRIPTION"));
        return n;
    }

    public synchronized AbstractGraph buildGraph(Object object) {
        if(object==null){
            return buildGraph();
        }
        if(object instanceof Project){
            return buildGraphProject((Project)object);
        } else if(object instanceof FileObject){
            return buildGraphFileObject((FileObject)object);
        } else {
            return null;
        }
    }

    public boolean acceptsNull() {
        return false;
    }

    private AbstractGraph buildGraph() {
        //TODO implement buildGraph() for AntGraphBuilder
        return null;
    }

    private AbstractGraph buildGraphProject(Project project) {
        try {
            FileObject folder = project.getProjectDirectory();
            FileObject fo = FileUtil.createData(folder, BUILDFILE);
            if(fo!=null){
                return buildGraphFileObject(fo);
            }
            return null;
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    private AbstractGraph buildGraphFileObject(FileObject fileObject) {
        try {
            AbstractGraphFactory f = AbstractGraphFactory.getFactory();
            AbstractGraph g = f.createAbstractGraph(fileObject.getName(), fileObject);
            AntProjectCookie antProjectCookie = AntScriptUtils.antProjectCookieFor(fileObject);
            Set<Target> targets = TargetLister.getTargets(antProjectCookie);
            Map<String, AbstractGraphNode> stringNodeMap = new HashMap<String, AbstractGraphNode>();
            for(Target target : targets){
                if(!target.isOverridden()){
                    Element element = target.getElement();
                    AbstractGraphNode node = f.createAbstractNode(target.getName(), g, fileObject);
                    node.addProperty(ANTDESCRIPTION, target.getElement().getAttribute(ANTDESCRIPTION));
                    node.addProperty(ANTIF, target.getElement().getAttribute(ANTIF));
                    node.addProperty(UNLESS, target.getElement().getAttribute(UNLESS));
                    NodeList tasks = element.getElementsByTagName(ASTERISK);
                    for (int i = 0; i < tasks.getLength(); i++) {
                        if (tasks.item(i).getParentNode().isSameNode(target.getElement())) {
                            node.addProperty(TASK+i, tasks.item(i).getNodeName());
                        }
                    }
                    stringNodeMap.put(target.getName(), node);
                }
            }
            for(Target target : targets){
                if(!target.isOverridden()){
                    AbstractGraphNode source = stringNodeMap.get(target.getName());
                    if(source!=null){
                        Element targetElement = target.getElement();
                        String dep = targetElement.getAttribute(DEPENDS);
                        StringTokenizer t = new StringTokenizer(dep, ",");
                        while (t.hasMoreTokens()) {
                            String s = t.nextToken().trim();
                            AbstractGraphNode destination = stringNodeMap.get(s);
                            if (destination == null) {
                                // not resolved
                                AbstractGraphNode n = f.createAbstractNode(s, g, fileObject);
                                n.addProperty(RESOLVED, "false");
                                AbstractGraphEdge edge = f.createAbstractEdge(edgeString(source, n), g, source, n, target.getScript().getFileObject());
                            } else {
                                AbstractGraphEdge edge = f.createAbstractEdge(edgeString(source, destination), g, source, destination, fileObject);
                            }
                        }
                    }
                }
            }
            return g;
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    private String edgeString(AbstractGraphNode from, AbstractGraphNode to){
        StringBuilder b = new StringBuilder();
        b.append(from.getName());
        b.append(TO);
        b.append(to.getName());
        return b.toString();
    }

    public boolean accepts(Object object) {
        if(object==null) return acceptsNull();
        return (Project.class.isAssignableFrom(object.getClass()) || FileObject.class.isAssignableFrom(object.getClass()));
    }

    public Collection<String> getMimeTypes() {
        List<String> l = new ArrayList<String>();
        l.add(new String(ANT));
        return Collections.unmodifiableCollection(l);
    }

}
