/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.classgraphbuilder;

import com.sun.source.tree.ClassTree;
import com.sun.source.util.TreePathScanner;
import com.sun.source.util.Trees;
import java.io.IOException;
import java.io.ObjectStreamField;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.TypeParameterElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.ArrayType;
import javax.lang.model.type.DeclaredType;
import javax.lang.model.type.ErrorType;
import javax.lang.model.type.PrimitiveType;
import javax.lang.model.type.TypeKind;
import javax.lang.model.type.TypeMirror;
import org.gpdv.services.GraphBuilder;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.AbstractGraphFactory;
import org.netbeans.api.java.source.ClasspathInfo;
import org.netbeans.api.java.source.CompilationController;
import org.netbeans.api.java.source.CompilationInfo;
import org.netbeans.api.java.source.ElementHandle;
import org.netbeans.api.java.source.JavaSource;
import org.netbeans.api.java.source.JavaSource.Phase;
import org.netbeans.api.java.source.SourceUtils;
import org.netbeans.api.java.source.Task;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphBuilder.class)

public class ClassGraphBuilderImpl implements GraphBuilder{

    private final String ANT = "text/x-java";

    private final String NAME = "CLASSGRAPHBUILDER";
    private final String DISPLAYNAME = "Graph Builder for Java Class Dependencies";
    private final String DESCRIPTION = "Graph Builder accepting DataObjects, FileObjects or JavaSources representing java-files as input to build an AbstractGraph";

    private final String MODIFIER = "Modifier";
    private final String PACKAGE = "PACKAGE";
    private final String PARAMETER = "PARAMETER";
    private final String SPACE = " : ";
    private final String TO = " -> ";
    private final String RESOLVED = "Resolved";

    private static ResourceBundle bundle = NbBundle.getBundle(ClassGraphBuilderImpl.class);

    private AbstractGraph graph;
    private int c;

    public Node getRepresentation() {
        Node n = new org.openide.nodes.AbstractNode(Children.LEAF);
        n.setName(bundle.getString("NAME"));
        n.setDisplayName(bundle.getString("DISPLAYNAME"));
        n.setShortDescription(bundle.getString("DESCRIPTION"));
        return n;
    }

    public synchronized AbstractGraph buildGraph(Object object) {
        if(object==null){
            return null;
        }
        if(object instanceof JavaSource){
            return buildGraphJavaSource((JavaSource)object);
        } else if(object instanceof DataObject){
            return buildGraphProject((DataObject)object);
        } else if(object instanceof FileObject){
            return buildGraphFileObject((FileObject)object);
        } else {
            return null;
        }
    }

    public boolean acceptsNull() {
        return false;
    }

    private AbstractGraph buildGraphJavaSource(JavaSource javaSource) {
        try {
            if (javaSource == null) {
                return null;
            }
            graph = null;
            final JavaSource source = javaSource;
            Thread t = new Thread(new Runnable() {

                public void run() {
                    try {
                        source.runUserActionTask(new Task<CompilationController>() {
                            public void run(CompilationController compilationController) throws Exception {
                                compilationController.toPhase(Phase.ELEMENTS_RESOLVED);
                                AbstractGraph g = new MemberVisitor(compilationController).scan(compilationController.getCompilationUnit(), null);
                                graph = g;
                            }
                        }, true);
                    } catch (IOException ex) {
                        Exceptions.printStackTrace(ex);
                    }
                }
            });
            t.start();
            t.join();
            return graph;
        } catch (InterruptedException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    private AbstractGraph buildGraphProject(DataObject dataObject) {
        return buildGraphFileObject(dataObject.getPrimaryFile());
    }

    private AbstractGraph buildGraphFileObject(FileObject fileObject) {
        return buildGraphJavaSource(JavaSource.forFileObject(fileObject));
    }

    public boolean accepts(Object object) {
        if(object==null) return acceptsNull();
        return (DataObject.class.isAssignableFrom(object.getClass()) || FileObject.class.isAssignableFrom(object.getClass()) || JavaSource.class.isAssignableFrom(object.getClass()));
    }

    public Collection<String> getMimeTypes() {
        List<String> l = new ArrayList<String>();
        l.add(new String(ANT));
        return Collections.unmodifiableCollection(l);
    }

    private class MemberVisitor extends TreePathScanner<AbstractGraph, Void> {

        private final int LEVELS = 4;

        private CompilationInfo info;

        public MemberVisitor(CompilationInfo info) {
            this.info = info;
        }

        @Override
        public AbstractGraph visitClass(ClassTree t, Void v) {
            Element el = info.getTrees().getElement(getCurrentPath());
            if (el == null) {
                // ERROR
            } else {
                AbstractGraphFactory factory = AbstractGraphFactory.getFactory();
                AbstractGraph g = factory.createAbstractGraph(info.getFileObject().getName(), null);
                Set<AbstractGraphNode> nodes = new HashSet<AbstractGraphNode>();
                Map<String, AbstractGraphNode> elementMap = new HashMap<String, AbstractGraphNode>();
                TypeElement te = (TypeElement) el;
                addElement(te, nodes, elementMap, g, factory, 0);
                return g;
            }
            return null;
        }

        private AbstractGraphNode addElement(Element te, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l){
            ElementHandle handle = ElementHandle.create(te);
            ClasspathInfo cpi = info.getClasspathInfo();
            FileObject fo = null;
            if(handle!=null&&cpi!=null){
                fo = SourceUtils.getFile(handle, cpi);
            }
            AbstractGraphNode n = f.createAbstractNode(te.getSimpleName().toString(), g, fo);
            nodes.add(n);
            elementMap.put(te.getSimpleName().toString(), n);
            if(l==LEVELS){
                return n;
            }
            List<? extends Element> enclosedElements = te.getEnclosedElements();
            for (Element enclosedElement : enclosedElements) {
                processEnclosedElement(enclosedElement, n, nodes, elementMap, g, f, l);
            }
            return n;
        }

        private void processDeclared(DeclaredType dt, Element parent, Modifier mod, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l, boolean param) {
            Element e = dt.asElement();
            AbstractGraphNode n1 = elementMap.get(e.getSimpleName().toString());
            if(n1!=null){
                // the returned type is already in graph
                if(n1.getName().equals(n.getName())){
                    // same element, do nothing
                } else {
                    // already defined
                    for(AbstractGraphEdge edge : g.getEdges()){
                        if(edge.getSource()==n&&edge.getDestination()==n1){
                            updateModifier(edge, mod);
                            updateParent(edge, parent, mod, param);
                            return;
                        }
                    }
                    StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                    b.append(TO);
                    b.append(e.getSimpleName().toString());
                    AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                    setModifier(edge, mod);
                    updateParent(edge, parent, mod, param);
                }
            } else {
                // the returned type is not in graph
                n1 = addElement(e, nodes, elementMap, g, f, l+1);
                elementMap.put(e.getSimpleName().toString(), n1);
                StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                b.append(TO);
                b.append(e.getSimpleName().toString());
                AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                setModifier(edge, mod);
                updateParent(edge, parent, mod, param);
            }
        }

        private void processTypeParameter(TypeParameterElement tpe, Element parent, Modifier mod, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l, boolean param) {
            TypeMirror m = tpe.asType();
            if(m.getKind().equals(TypeKind.DECLARED)){
                processDeclared((DeclaredType)m, parent, mod, n, nodes, elementMap, g, f, l, true);
            } else if(m.getKind().isPrimitive()){
                processPrimitive((PrimitiveType)m, parent, mod, n, nodes, elementMap, g, f, l, true);
            }
        }

        private void processArray(ArrayType ar, Element parent, Modifier mod, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l, boolean param) {
            TypeMirror art = ar.getComponentType();
            if(art.getKind().equals(TypeKind.DECLARED)){
                processDeclared((DeclaredType)art, parent, mod, n, nodes, elementMap, g, f, l, param);
            } else if(art.getKind().isPrimitive()){
                processPrimitive((PrimitiveType)art, parent, mod, n, nodes, elementMap, g, f, l, param);
            }
        }

        private void processPrimitive(PrimitiveType dt, Element parent, Modifier mod, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l, boolean param) {

            AbstractGraphNode n1 = elementMap.get(dt.getKind().toString());
            if(n1!=null){
                // the returned type is already in graph
                if(n1.getName().equals(n.getName())){
                    // same element, do nothing
                } else {
                    // already defined
                    for(AbstractGraphEdge edge : g.getEdges()){
                        if(edge.getSource()==n&&edge.getDestination()==n1){
                            updateModifier(edge, mod);
                            updateParent(edge, parent, mod, param);
                            return;
                        }
                    }
                    StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                    b.append(TO);
                    b.append(dt.getKind().toString());
                    AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                    setModifier(edge, mod);
                    updateParent(edge, parent, mod, param);
                }
            } else {
                // the returned type is not in graph
                n1 = f.createAbstractNode(dt.getKind().toString(), g, null);
                elementMap.put(dt.getKind().toString(), n1);
                StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                b.append(TO);
                b.append(dt.getKind().toString());
                AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                setModifier(edge, mod);
                updateParent(edge, parent, mod, param);
            }
        }

        private void processError(ErrorType dt, Element parent, Modifier mod, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l, boolean param) {
            AbstractGraphNode n1 = elementMap.get(dt.toString());
            if(n1!=null){
                // the returned type is already in graph
                if(n1.getName().equals(n.getName())){
                    // same element, do nothing
                } else {
                    // already defined
                    for(AbstractGraphEdge edge : g.getEdges()){
                        if(edge.getSource()==n&&edge.getDestination()==n1){
                            updateModifier(edge, mod);
                            updateParent(edge, parent, mod, param);
                            return;
                        }
                    }
                    StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                    b.append(TO);
                    b.append(dt.getKind().toString());
                    AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                    setModifier(edge, mod);
                    updateParent(edge, parent, mod, param);
                }
            } else {
                // the returned type is not in graph
                n1 = f.createAbstractNode(dt.toString(), g, null);
                n1.addProperty(RESOLVED, "false");
                elementMap.put(dt.toString(), n1);
                StringBuffer b = new StringBuffer(parent.getSimpleName().toString());
                b.append(TO);
                b.append(dt.getKind().toString());
                AbstractGraphEdge edge = f.createAbstractEdge(b.toString(), g, n, n1, null);
                setModifier(edge, mod);
                updateParent(edge, parent, mod, param);
            }
        }

        private void processEnclosedElement(Element enclosedElement, AbstractGraphNode n, Set<AbstractGraphNode> nodes, Map<String, AbstractGraphNode> elementMap, AbstractGraph g, AbstractGraphFactory f, int l) {
            Set<Modifier> mods = enclosedElement.getModifiers();
            Modifier mod = null;
            for(Modifier m : mods){
                if(m==Modifier.PROTECTED&&mod==null) mod = m;
                if(m==Modifier.PUBLIC){
                    mod = m;
                    break;
                }
            }
            if (enclosedElement.getKind() == ElementKind.METHOD || enclosedElement.getKind().equals(ElementKind.CONSTRUCTOR)) {
                    // do something with method or constructor
                    ExecutableElement executable = ((ExecutableElement)enclosedElement);
                    if(executable.getModifiers().contains(Modifier.PRIVATE)){
                        return;
                    }
                    TypeMirror returnType = executable.getReturnType();
                    if(returnType.getKind().equals(TypeKind.DECLARED)){
                        processDeclared((DeclaredType)returnType, executable, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(returnType.getKind().equals(TypeKind.ARRAY)){
                        processArray((ArrayType)returnType, executable, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(returnType.getKind().isPrimitive()){
                        processPrimitive((PrimitiveType)returnType, executable, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(returnType.getKind().equals(TypeKind.ERROR)){
                        processError((ErrorType)returnType, executable, mod, n, nodes, elementMap, g, f, l, false);
                    }
                    List<? extends VariableElement> parameters = executable.getParameters();
                    for(VariableElement ve : parameters){
                        if(ve.getModifiers().contains(Modifier.PRIVATE)){
                            return;
                        }
                        TypeMirror tm = ve.asType();
                        if(tm.getKind().equals(TypeKind.DECLARED)){
                            processDeclared((DeclaredType)tm, executable, mod, n, nodes, elementMap, g, f, l, true);
                        } else if(tm.getKind().equals(TypeKind.ARRAY)){
                            processArray((ArrayType)tm, executable, mod, n, nodes, elementMap, g, f, l, true);
                        } else if(tm.getKind().isPrimitive()){
                            processPrimitive((PrimitiveType)tm, executable, mod, n, nodes, elementMap, g, f, l, true);
                        } else if(tm.getKind().equals(TypeKind.ERROR)){
                            processError((ErrorType)tm, executable, mod, n, nodes, elementMap, g, f, l, true);
                        }
                    }
                    List<? extends TypeMirror> thrownParameters = executable.getThrownTypes();
                    for(TypeMirror tm : thrownParameters){
                        if(tm.getKind().equals(TypeKind.DECLARED)){
                            processDeclared((DeclaredType)tm, executable, mod, n, nodes, elementMap, g, f, l, false);
                        } else if(tm.getKind().isPrimitive()){
                            processPrimitive((PrimitiveType)tm, executable, mod, n, nodes, elementMap, g, f, l, false);
                        } else if(tm.getKind().equals(TypeKind.ERROR)){
                            processError((ErrorType)tm, executable, mod, n, nodes, elementMap, g, f, l, false);
                        }
                    }
                } else if(enclosedElement.getKind() == ElementKind.FIELD) {
                    // do something with field
                    VariableElement ve = (VariableElement)enclosedElement;
                    if(ve.getModifiers().contains(Modifier.PRIVATE)){
                        return;
                    }
                    TypeMirror tm = ve.asType();
                    if(tm.getKind().equals(TypeKind.DECLARED)){
                        processDeclared((DeclaredType)tm, ve, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(tm.getKind().equals(TypeKind.ARRAY)){
                        processArray((ArrayType)tm, ve, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(tm.getKind().isPrimitive()){
                        processPrimitive((PrimitiveType)tm, ve, mod, n, nodes, elementMap, g, f, l, false);
                    } else if(tm.getKind().equals(TypeKind.ERROR)){
                        processError((ErrorType)tm, ve, mod, n, nodes, elementMap, g, f, l, false);
                    }
                } else {
                }
        }

        private void updateModifier(AbstractGraphEdge edge, Modifier mod) {
            if(mod!=null){
                String modifier = edge.getValue(MODIFIER);
                if(modifier==null){
                    edge.addProperty(MODIFIER, mod.name());
                } else if(!modifier.contains(mod.name())){
                    StringBuilder b = new StringBuilder(modifier);
                    b.append(", ");
                    b.append(mod.name());
                    edge.removeProperty(MODIFIER);
                    edge.addProperty(MODIFIER, b.toString());
                }
            } else {
                String modifier = edge.getValue(MODIFIER);
                if(modifier==null){
                    edge.addProperty(MODIFIER, PACKAGE);
                } else if(!modifier.contains(PACKAGE)){
                    StringBuilder b = new StringBuilder(modifier);
                    b.append(", ");
                    b.append(PACKAGE);
                    edge.removeProperty(MODIFIER);
                    edge.addProperty(MODIFIER, b.toString());
                }
            }
        }

        private void setModifier(AbstractGraphEdge edge, Modifier mod) {
            if(mod!=null){
                edge.addProperty(MODIFIER, mod.name());
            } else {
                edge.addProperty(MODIFIER, PACKAGE);
            }
        }

        private void updateParent(AbstractGraphEdge edge, Element parent, Modifier mod, boolean param) {
            if(parent==null) return;
            StringBuilder sb = new StringBuilder();
            String s;
            if(mod!=null){
                sb.append(mod.name());
            } else {
                sb.append(PACKAGE);
            }
            s = sb.toString();
            StringBuilder sb1 = new StringBuilder();
            sb1.append(parent.getSimpleName());
            sb1.append(SPACE);
            sb1.append(parent.getKind().name());
            if(param){
                sb1.append(" ");
                sb1.append(PARAMETER);
            }
            String name = sb1.toString();
            edge.addProperty(name, s);
        }

    }

}
