/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.defaultgraphdesigner;

import java.awt.Color;
import java.awt.Image;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.ResourceBundle;
import java.util.Set;
import org.gpdv.services.GraphDesigner;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.api.designgraph.DesignGraphEdge;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.designgraph.DesignGraphNode;
import org.gpdv.core.api.designgraph.DesignGraphSlot;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphDesigner.class)
public class ClassGraphDesigner implements GraphDesigner{

    private static ResourceBundle bundle = NbBundle.getBundle(ClassGraphDesigner.class);

    public boolean designGraph(DesignGraph graph) {
        if(graph==null) return false;
        for (DesignGraphNode n : graph.getDesignNodes()) {
            for (DesignGraphSlot s : n.getIncomingSlots()) {
                s.setDesign(new SlotDesign());
            }
            for(DesignGraphSlot s : n.getOutgoingSlots()){
                s.setDesign(new SlotDesign());
            }
            designNode(n);
        }
        for(DesignGraphEdge e : graph.getDesignEdges()){
            e.setDesign(new EdgeDesign());
        }
        return true;
    }

    public boolean designEdge(DesignGraphEdge edge) {
        if(edge==null) return false;
        edge.setDesign(new EdgeDesign());
        return true;
    }

    public boolean designNode(DesignGraphNode node) {
        if(node==null) return false;
        org.gpdv.core.api.abstractgraph.AbstractGraphNode n = node.getAbstractNode();
        String s = n.getValue(bundle.getString("resolved"));
        if(s!=null&&s.equals("false")){
            node.setDesign(getErrorDesign(node));
        } else {
            node.setDesign(getNodeDesign(node));
        }
        return true;
    }

    public boolean designSlot(DesignGraphSlot slot) {
        if(slot==null) return false;
        slot.setDesign(new SlotDesign());
        return true;
    }

    public Node getRepresentation() {
        Node n = new AbstractNode(Children.LEAF);
        n.setName(bundle.getString("classname"));
        n.setDisplayName(bundle.getString("classdisplayname"));
        n.setShortDescription(bundle.getString("classdescription"));
        return n;
    }

    public Collection<String> getMimeTypes() {
        Set<String> s = new HashSet<String>();
        s.add(bundle.getString("class"));
        return Collections.unmodifiableCollection(s);
    }


    private final class SlotDesign extends Design{
        public SlotDesign(){
            super(Color.BLACK, Color.GREEN, 5, 5, null);
        }
    }

    private final class NodeDesign extends Design{
        public NodeDesign(DesignGraphNode node){
            super(Color.ORANGE, Color.GREEN, 30, node.getName().length()*10, null);
        }
    }

    private final class EdgeDesign extends Design{
        public EdgeDesign(){
            super(Color.BLACK, Color.GREEN, 0, 0);
        }
    }

    private Design getErrorDesign(DesignGraphNode n){
        Image image = (ImageUtilities.loadImage(bundle.getString("error"))).getScaledInstance(-1, 50, Image.SCALE_FAST);
        int width = Math.max(n.getName().length()*10, image.getWidth(null));
        int height = 80;
        Color[] l = new Color[10];
        l[Design.BACKGROUND_POSITION] = new Color(200,0,0);
        l[Design.INCREASED_BACKGROUND_POSITION] = new Color(255,0,0);
        l[Design.BACKGROUND_HOVERED_POSITION] = Color.YELLOW;
        l[Design.FONT_POSITION] = Color.WHITE;
        int[] v = Design.getDefaultValues();
        v[Design.SELECTIONSIZEINCREASE_HORIZONTAL_POSITION] = 5;
        v[Design.SELECTIONSIZEINCREASE_VERTICAL_POSITION] = 5;
        v[Design.ARCWIDTH_POSITION] = 0;
        v[Design.ARCHEIGHT_POSITION] = 0;
        v[Design.ARCWIDTH_MINIMIZED_POSITION] = 30;
        v[Design.ARCHEIGHT_MINIMIZED_POSITION] = 30;
        v[Design.MINIMIZEDWIDTH_POSITION] = 30;
        v[Design.MINIMIZEDHEIGHT_POSITION] = 30;
        return new Design(l, v, height, width, image);
    }

    private Design getNodeDesign(DesignGraphNode n){
        int width = n.getName().length()*10;
        int height = 30;
        Color[] l = new Color[10];
        l[Design.BACKGROUND_POSITION] = Color.orange;
        l[Design.INCREASED_BACKGROUND_POSITION] = Color.lightGray;
        l[Design.BACKGROUND_HOVERED_POSITION] = Color.YELLOW;
        l[Design.BACKGROUND_SELECTED_POSITION] = Color.green;
        int[] v = Design.getDefaultValues();
        v[Design.SELECTIONSIZEINCREASE_HORIZONTAL_POSITION] = 5;
        v[Design.SELECTIONSIZEINCREASE_VERTICAL_POSITION] = 5;
        v[Design.ARCWIDTH_POSITION] = 0;
        v[Design.ARCHEIGHT_POSITION] = 0;
        v[Design.ARCWIDTH_MINIMIZED_POSITION] = 10;
        v[Design.ARCHEIGHT_MINIMIZED_POSITION] = 10;
        v[Design.MINIMIZEDWIDTH_POSITION] = 15;
        v[Design.MINIMIZEDHEIGHT_POSITION] = 15;
        return new Design(l, v, height, width, null);
    }

}
