/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gpdv.services.defaultgraphdesigner;

import java.awt.Color;
import java.awt.Image;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.ResourceBundle;
import java.util.Set;
import org.gpdv.services.GraphDesigner;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.api.designgraph.DesignGraphEdge;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.designgraph.DesignGraphNode;
import org.gpdv.core.api.designgraph.DesignGraphSlot;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.lookup.ServiceProvider;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;


/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphDesigner.class)
public class DefaultGraphDesigner implements GraphDesigner {

    private static ResourceBundle bundle = NbBundle.getBundle(DefaultGraphDesigner.class);

    public boolean designGraph(DesignGraph graph) {
        if(graph==null) return false;
        for (DesignGraphNode n : graph.getDesignNodes()) {
            for (DesignGraphSlot s : n.getIncomingSlots()) {
                s.setDesign(new SlotDesign());
            }
            for(DesignGraphSlot s : n.getOutgoingSlots()){
                s.setDesign(new SlotDesign());
            }
            n.setDesign(new NodeDesign(n));
        }
        for(DesignGraphEdge e : graph.getDesignEdges()){
            e.setDesign(new EdgeDesign());
        }
        return true;
    }

    public boolean designEdge(DesignGraphEdge edge) {
        if(edge==null) return false;
        edge.setDesign(new EdgeDesign());
        return true;
    }

    public boolean designNode(DesignGraphNode node) {
        if(node==null) return false;
        node.setDesign(new NodeDesign(node));
        return true;
    }

    public boolean designSlot(DesignGraphSlot slot) {
        if(slot==null) return false;
        slot.setDesign(new SlotDesign());
        return true;
    }

    public Node getRepresentation() {
        Node n = new AbstractNode(Children.LEAF);
        n.setName(bundle.getString("defaultname"));
        n.setDisplayName(bundle.getString("defaultdisplayname"));
        n.setShortDescription(bundle.getString("defaultdescription"));
        return n;
    }

    public Collection<String> getMimeTypes() {
        Set<String> s = new HashSet<String>();
        s.add(bundle.getString("universal"));
        return Collections.unmodifiableCollection(s);
    }


    private final class SlotDesign extends Design{
        public SlotDesign(){
            super(Color.BLACK, Color.GREEN, 5, 5, null);
        }
    }

    private final class NodeDesign extends Design{
        public NodeDesign(DesignGraphNode node){
            super(Color.ORANGE, Color.GREEN, 50, node.getName().length()*10, null);
        }
    }

    private final class EdgeDesign extends Design{
        public EdgeDesign(){
            super(Color.BLACK, Color.GREEN, 0, 0);
        }
    }
}
