/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.defaultgraphlayouter;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import org.gpdv.services.GraphLayouter;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdge;
import org.gpdv.core.api.layoutgraph.LayoutGraph;
import org.gpdv.core.api.layoutgraph.LayoutGraphNode;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlot;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphLayouter.class)

public class DefaultGraphLayouter implements GraphLayouter{

//    private final String NAME = "DEFAULTGRAPHDLAYOUTER";
//    private final String DISPLAYNAME = "Force-based layouter";
//    private final String DESCRIPTION = "Force-based layouter included in GPDV bundle";

    private final int ITERATIONS = 10000;
    private final int MAXMOVE = 5;
    private final int DAMPING = 2;

    private static ResourceBundle bundle = NbBundle.getBundle(DefaultGraphLayouter.class);

    public synchronized boolean layoutGraph(LayoutGraph graph) {
        if(graph==null) return false;
        List<Vertex> vertices = new ArrayList<Vertex>();
        List<Edge> edges = new ArrayList<Edge>();
        getVertices(vertices, edges, graph.getLayoutNodes(), graph.getLayoutEdges());
        double width = vertices.size();
        double height = vertices.size();
        for(Vertex vertex : vertices){
            double x;
            double y;
            x = (Math.random() * width);
            y = (Math.random() * height);
            vertex.posX = x;
            vertex.posY = y;
        }
        for(int i = 0; i < ITERATIONS; i++){
            for (Vertex v : vertices) {
                double dx = 0;
                double dy = 0;
                for (Vertex u : vertices) {
                    if (v!=u) {
                        double velX = v.posX - u.posX;
                        double velY = v.posY - u.posY;
                        double len = velX * velX + velY * velY;
                        if (len == 0) {
                            dx += Math.random();
                            dy += Math.random();
                        } else if (len < 10000) {
                            dx += velX / len;
                            dy += velY / len;
                        }
                    }
                }
                double dlen = dx * dx + dy * dy;
                if (dlen > 0) {
                    dlen = Math.sqrt(dlen) / 2;
                    v.dispX += dx / dlen;
                    v.dispY += dy / dlen;
                }
            }

            for (Edge e : edges) {
                double velX = e.destination.posX - e.source.posX;
                double velY = e.destination.posY - e.source.posY;
                double len = Math.sqrt(velX * velX + velY * velY);
                if(len==0) len = 0.0001;
                double force = (e.len - len) / (len * 3);
                double dx = force * velX;
                double dy = force * velY;

                e.destination.dispX += dx;
                e.destination.dispY += dy;
                e.source.dispX += -dx;
                e.source.dispY += -dy;
            }

            for (Vertex v : vertices) {
                v.posX += Math.max(-MAXMOVE, Math.min(MAXMOVE, v.dispX));
                v.posY += Math.max(-MAXMOVE, Math.min(MAXMOVE, v.dispY));
                if (v.posX < 0) {
                    v.posX= 0;
                } else if (v.posX > width) {
                    v.posX = width;
                }
                if (v.posY < 0) {
                    v.posY = 0;
                } else if (v.posY > height) {
                    v.posY = height;
                }
                v.dispX /= DAMPING;
                v.dispY /= DAMPING;
            }
        }
        int maxWidth = maxWidth(graph.getLayoutNodes());
        int maxHeight = maxHeight(graph.getLayoutNodes());
        move(vertices, width, height);
        for(Vertex v : vertices){
            LayoutGraphNode n = v.node;
            Point p = new Point((int)v.posX*maxWidth, (int)v.posY*maxHeight);
            n.setPoint(p);
            setSlots(n);
        }
        return true;
    }

    private void move(List<Vertex> vertices, double width, double height){
        double minX = width;
        double minY = height;
        for(Vertex v : vertices){
            if(v.posX<minX) minX = v.posX;
            if(v.posY<minY) minY = v.posY;
        }
        for(Vertex v : vertices){
            v.posX -= minX;
            v.posY -= minY;
        }
    }

    private int maxWidth(List<LayoutGraphNode> list){
        int c = 0;
        for(LayoutGraphNode n : list){
            c += n.getWidth();
        }
        c = c / list.size();
        return c;
    }

    private int maxHeight(List<LayoutGraphNode> list){
        int i = 0;
        for(LayoutGraphNode n : list){
            i += n.getHeight();
        }
        i = i / list.size();
        return i;
    }

    private void getVertices(List<Vertex> vertices, List<Edge> edges, List<LayoutGraphNode> gnodes, List<LayoutGraphEdge> gedges){
        vertices.clear();
        edges.clear();
        Map<LayoutGraphNode, Vertex> map = new HashMap<LayoutGraphNode, Vertex>();
        for(LayoutGraphNode n : gnodes){
            Vertex v = new Vertex(n);
            map.put(n, v);
            vertices.add(v);
        }
        for(LayoutGraphEdge e : gedges){
            Vertex s = map.get(e.getSourceLayoutSlot().getLayoutNode());
            Vertex d = map.get(e.getDestinationLayoutSlot().getLayoutNode());
            Edge ed = new Edge(s, d);
            edges.add(ed);
        }
    }

    public Node getRepresentation() {
        Node n = new AbstractNode(Children.LEAF);
        n.setName(bundle.getString("name"));
        n.setDisplayName(bundle.getString("displayname"));
        n.setShortDescription(bundle.getString("description"));
        return n;
    }

    private void setSlots(LayoutGraphNode node) {
        Point p = node.getPoint();
        int i = 0;
        int count = node.getIncomingLayoutSlots().size();
        for(LayoutGraphSlot s : node.getIncomingLayoutSlots()){
            s.setPoint(new Point((p.x+(node.getWidth()/count*i)), (p.y-s.getHeight())));
            i++;
        }
        i = 0;
        count = node.getOutgoingLayoutSlots().size();
        for(LayoutGraphSlot s : node.getOutgoingLayoutSlots()){
            s.setPoint(new Point((p.x+(node.getWidth()/count*i)), (p.y+node.getHeight())));
            i++;
        }
    }

    public boolean isSlotAware() {
        return true;
    }

    @Override
    public String toString() {
        return bundle.getString("displayname");
    }

}
