/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.api.designgraph.DesignGraphEdge;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.designgraph.DesignGraphNode;
import org.gpdv.core.api.designgraph.DesignGraphSlot;
import org.openide.util.Exceptions;

/**
 *
 * @author odi
 */
public final class DesignGraphFactory {

    private final Color DEFAULT_COLOR = Color.WHITE;
    private final int HEIGHT = 100;
    private final int SLOTSIZE = 20;
    private long graphID;
    private long nodeID;
    private long edgeID;
//    private long slotID;
    private static DesignGraphFactory instance;

    private DesignGraphFactory(){
        graphID = 0;
        nodeID = 0;
        edgeID = 0;
//        slotID = 0;
    }

    public static synchronized DesignGraphFactory getFactory(){
        if(instance==null) instance = new DesignGraphFactory();
        return instance;
    }

    private long getGraphID() {
        return graphID++;
    }

    private long getNodeID() {
        return nodeID++;
    }

    private long getEdgeID() {
        return edgeID++;
    }

    private int getWidth(String n){
        return n.length()*20;
    }

    public synchronized DesignGraph convertAbstractToDesignGraph(AbstractGraph graph){
        DesignGraph dg = createDesignGraph(graph);
        Map<AbstractGraphNode, DesignGraphNode> map = new HashMap<AbstractGraphNode, DesignGraphNode>();
        // create all nodes and add incoming and outgoing slots, save corresponding nodes in map
        for(AbstractGraphNode n : graph.getNodes()){
            DesignGraphNode dn = createDesignNode(n, dg, new Design(DEFAULT_COLOR,DEFAULT_COLOR, HEIGHT, getWidth(n.getName())));
            createIncomingSlot(dn);
            createOutgoingSlot(dn);
            map.put(n, dn);
        }
        for(AbstractGraphEdge e : graph.getEdges()){
            DesignGraphNode source = map.get(e.getSource());
            DesignGraphNode destination = map.get(e.getDestination());
            DesignGraphEdge ed = createDesignEdge(e, dg, source.getOutgoingSlots().get(0), destination.getIncomingSlots().get(0), new Design(DEFAULT_COLOR,DEFAULT_COLOR, 0, 0));
        }
        return dg;
    }

    public synchronized DesignGraph createDesignGraph(AbstractGraph graph){
        return new DesignGraph(getGraphID(), graph);
    }

    public synchronized DesignGraphNode createDesignNode(AbstractGraphNode node, DesignGraph graph, Design design){
        try {
            return new DesignGraphNode(getNodeID(), node, graph, design);
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    public synchronized DesignGraphEdge createDesignEdge(AbstractGraphEdge edge, DesignGraph graph, DesignGraphSlot source, DesignGraphSlot destination, Design design){
        try {
            return new DesignGraphEdge(getEdgeID(), edge, graph, source, destination, design);
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    public synchronized DesignGraphSlot createIncomingSlot(DesignGraphNode node){
        try {
            return new DesignGraphSlot(node, DesignGraphSlot.INCOMING, new Design(DEFAULT_COLOR, DEFAULT_COLOR, SLOTSIZE, SLOTSIZE));
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    public synchronized DesignGraphSlot createOutgoingSlot(DesignGraphNode node){
        try {
            return new DesignGraphSlot(node, DesignGraphSlot.OUTGOING, new Design(DEFAULT_COLOR, DEFAULT_COLOR, SLOTSIZE, SLOTSIZE));
        } catch (Exception ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }
}
