/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;

/**
 *
 * @author odi
 */
public final class GraphUtils {

    private static GraphUtils instance;

    private GraphUtils(){
    }

    public static synchronized GraphUtils getInstance() {
        if(instance==null) instance = new GraphUtils();
        return instance;
    }

    public static Map<AbstractGraphNode, List<AbstractGraphNode>> getSuccessors(AbstractGraph graph) {
        HashMap<AbstractGraphNode, List<AbstractGraphNode>> succ = new HashMap<AbstractGraphNode, List<AbstractGraphNode>>();
        ListIterator i = graph.getNodes().listIterator();
        AbstractGraphNode n;
        while(i.hasNext()){
            n = (AbstractGraphNode) i.next();
            succ.put(n, GraphUtils.getSuccessors(n));
        }
        return Collections.unmodifiableMap(succ);
    }

    public static Map<AbstractGraphNode, List<AbstractGraphNode>> getPredecessors(AbstractGraph graph) {
        HashMap<AbstractGraphNode, List<AbstractGraphNode>> pre = new HashMap<AbstractGraphNode, List<AbstractGraphNode>>();
        ListIterator i = graph.getNodes().listIterator();
        AbstractGraphNode n;
        while(i.hasNext()){
            n = (AbstractGraphNode) i.next();
            pre.put(n, GraphUtils.getPredecessors(n));
        }
        return Collections.unmodifiableMap(pre);
    }

    public static Map<AbstractGraphNode, List<AbstractGraphEdge>> getIncomingEdges(AbstractGraph graph) {
        HashMap<AbstractGraphNode, List<AbstractGraphEdge>> incEdges = new HashMap<AbstractGraphNode, List<AbstractGraphEdge>>();
        ListIterator i = graph.getNodes().listIterator();
        AbstractGraphNode n;
        while(i.hasNext()){
            n = (AbstractGraphNode) i.next();
            incEdges.put(n, n.getIncomingEdges());
        }
        return Collections.unmodifiableMap(incEdges);
    }

    public static Map<AbstractGraphNode, List<AbstractGraphEdge>> getOutgoingEdges(AbstractGraph graph) {
        HashMap<AbstractGraphNode, List<AbstractGraphEdge>> outEdges = new HashMap<AbstractGraphNode, List<AbstractGraphEdge>>();
        ListIterator i = graph.getNodes().listIterator();
        AbstractGraphNode n;
        while(i.hasNext()){
            n = (AbstractGraphNode) i.next();
            outEdges.put(n, n.getOutgoingEdges());
        }
        return Collections.unmodifiableMap(outEdges);
    }

    public static List<AbstractGraphNode> getSuccessors(AbstractGraphNode node) {
        ArrayList<AbstractGraphNode> succ = new ArrayList<AbstractGraphNode>();
        Iterator i = node.getOutgoingEdges().iterator();
        while(i.hasNext()){
            succ.add(((AbstractGraphEdge)i.next()).getDestination());
        }
        return Collections.unmodifiableList(succ);
    }

    public static List<AbstractGraphNode> getPredecessors(AbstractGraphNode node) {
        ArrayList<AbstractGraphNode> pre = new ArrayList<AbstractGraphNode>();
        Iterator i = node.getIncomingEdges().iterator();
        while(i.hasNext()){
            pre.add(((AbstractGraphEdge)i.next()).getSource());
        }
        return Collections.unmodifiableList(pre);
    }

    public static AbstractGraphNode getNode(AbstractGraph graph, long id){
        Iterator i = graph.getNodes().iterator();
        AbstractGraphNode n;
        while(i.hasNext()){
            n =   (AbstractGraphNode) i.next();
            if(n.getID()==id) return n;
        }
        return null;
    }

    public static AbstractGraphEdge getEdge(AbstractGraph graph, long id){
        Iterator i = graph.getEdges().iterator();
        AbstractGraphEdge e;
        while(i.hasNext()){
            e =   (AbstractGraphEdge) i.next();
            if(e.getID()==id) return e;
        }
        return null;
    }

    public static AbstractGraphNode getNode(AbstractGraph graph, String name){
        Iterator i = graph.getNodes().iterator();
        AbstractGraphNode e;
        while(i.hasNext()){
            e =   (AbstractGraphNode) i.next();
            if(e.getName().equals(name)) return e;
        }
        return null;
    }

    public static AbstractGraphEdge getEdge(AbstractGraph graph, String name){
        Iterator i = graph.getEdges().iterator();
        AbstractGraphEdge e;
        while(i.hasNext()){
            e =   (AbstractGraphEdge) i.next();
            if(e.getName().equals(name)) return e;
        }
        return null;
    }
}
