/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api;

import java.awt.Color;
import java.util.List;
import org.gpdv.services.GraphDesigner;
import org.gpdv.services.GraphLayouter;
import org.gpdv.services.validators.AbstractGraphValidator;
import org.gpdv.services.validators.DesignGraphValidator;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.services.GraphRouter;

/**
 *
 * @author odi
 */
public class VisualizationRequest {

    private AbstractGraph abstractGraph;
    private DesignGraph designGraph;
    private List<AbstractGraphValidator> aGraphValidators;
    private List<DesignGraphValidator> dGraphValidators;
    private List<GraphDesigner> designers;
    private List<GraphLayouter> layouter;
    private List<GraphRouter> routers;

    private GraphDesigner preferredDesigner;
    private GraphLayouter preferredLayouter;

    private boolean toolbar;
    private boolean allowResetLayout;
    private boolean allowRedesign;
    private boolean allowRelayouting;
    private boolean allowDifferentDesigner;
    private boolean allowDifferentLayouter;
    private boolean movement;
    private boolean pins;
    private boolean pinMovement;
    private boolean allowDifferentValidators;
    private boolean allowDifferentRouter;
    private boolean standalone;
    private GraphRouter router;
    private Color background;
    private List<AbstractGraphValidator> preferredAbstractGraphValidators;
    private List<DesignGraphValidator> preferredDesignGraphValidators;

    /**
     * @return true if resetting all nodes to their original position is allowed
     */
    public boolean allowResetLayout() {
        return allowResetLayout;
    }

    public VisualizationRequest allowResetLayout(boolean allowResetLayout) {
        this.allowResetLayout = allowResetLayout;
        return this;
    }

    /**
     * @return the GraphRouter that should be used
     */
    public GraphRouter getRouter() {
        return router;
    }

    public VisualizationRequest setRouter(GraphRouter router) {
        this.router = router;
        return this;
    }

    /**
     * @return true if different routers are allowed
     */
    public boolean allowDifferentRouter(){
        return allowDifferentRouter;
    }

    public VisualizationRequest allowDifferentRouter(boolean allowDifferentRouter){
        this.allowDifferentRouter = allowDifferentRouter;
        return this;
    }

    /**
     * @return the list of available routers
     */
    public List<GraphRouter> getRouters(){
        return routers;
    }

    public VisualizationRequest setRouters(List<GraphRouter> routers){
        this.routers = routers;
        return this;
    }

    /**
     * @return true if Pins/Slots should be allowed
     */
    public boolean allowPins() {
        return pins;
    }

    public VisualizationRequest allowPins(boolean allowPins) {
        this.pins = allowPins;
        return this;
    }

    /**
     * @return the designer that should be used
     */
    public GraphDesigner getPreferredDesigner() {
        return preferredDesigner;
    }

    public VisualizationRequest setPreferredDesigner(GraphDesigner preferredDesigner) {
        this.preferredDesigner = preferredDesigner;
        return this;
    }

    /**
     * @return the layouter that should be used
     */
    public GraphLayouter getPreferredLayouter() {
        return preferredLayouter;
    }

    public VisualizationRequest setPreferredLayouter(GraphLayouter preferredLayouter) {
        this.preferredLayouter = preferredLayouter;
        return this;
    }

    /**
     * @return true if Slots/Pins should be moveable independently
     */
    public boolean allowPinMovement(){
        return this.pinMovement;
    }

    public VisualizationRequest allowPinMovement(boolean allowPinMovement){
        this.pinMovement = allowPinMovement;
        return this;
    }

    /**
     * @return true if the component based on this request should be a standalone visualizer
     */
    public boolean standalone() {
        return standalone;
    }

    public VisualizationRequest standalone(boolean standalone) {
        this.standalone = standalone;
        return this;
    }

    /**
     * @return true if validators can be changed
     */
    public boolean allowDifferentValidator() {
        return allowDifferentValidators;
    }

    public VisualizationRequest allowDifferentValidators(boolean allowDifferentValidators) {
        this.allowDifferentValidators = allowDifferentValidators;
        return this;
    }

    /**
     * @return true if rerunning the layouter is allowed
     */
    public boolean allowRelayouting() {
        return allowRelayouting;
    }

    public VisualizationRequest allowRelayouting(boolean allowRelayouting) {
        this.allowRelayouting = allowRelayouting;
        return this;
    }

    /**
     * @return the list of allowed AbstractGraphValidators
     */
    public List<AbstractGraphValidator> getAbstractGraphValidators() {
        return aGraphValidators;
    }

    public VisualizationRequest setAbstractGraphValidators(List<AbstractGraphValidator> aGraphValidators) {
        this.aGraphValidators = aGraphValidators;
        return this;
    }

    /**
     * @return true if setting a different designer is allowed
     */
    public boolean allowDifferentDesigner() {
        return allowDifferentDesigner;
    }

    public VisualizationRequest allowDifferentDesigner(boolean allowDifferentDesigner) {
        this.allowDifferentDesigner = allowDifferentDesigner;
        return this;
    }

    /**
     * @return true if setting a different layouter is allowed
     */
    public boolean allowDifferentLayouter() {
        return allowDifferentLayouter;
    }

    public VisualizationRequest allowDifferentLayouter(boolean allowDifferentLayouter) {
        this.allowDifferentLayouter = allowDifferentLayouter;
        return this;
    }

    /**
     * @return true if nodes should be moveable
     */
    public boolean allowMovement() {
        return movement;
    }

    public VisualizationRequest allowMovement(boolean allowMovement) {
        this.movement = allowMovement;
        return this;
    }

    /**
     * @return true if redesigning is allowed
     */
    public boolean allowRedesign() {
        return allowRedesign;
    }

    public VisualizationRequest allowRedesign(boolean allowRedesign) {
        this.allowRedesign = allowRedesign;
        return this;
    }

    /**
     * @return the list of allowed DesignGraphValidators
     */
    public List<DesignGraphValidator> getDesignGraphValidators() {
        return dGraphValidators;
    }

    public VisualizationRequest setDesignGraphValidators(List<DesignGraphValidator> dGraphValidators) {
        this.dGraphValidators = dGraphValidators;
        return this;
    }

    /**
     * @return the list of available GraphDesigners
     */
    public List<GraphDesigner> getDesigners() {
        return designers;
    }

    public VisualizationRequest setDesigners(List<GraphDesigner> designers) {
        this.designers = designers;
        return this;
    }

    /**
     * @return the available GraphLayouters
     */
    public List<GraphLayouter> getLayouter() {
        return layouter;
    }

    public VisualizationRequest setLayouter(List<GraphLayouter> layouter) {
        this.layouter = layouter;
        return this;
    }

    /**
     * @return true if a allowToolbar can be retrieved
     */
    public boolean allowToolbar() {
        return toolbar;
    }

    public VisualizationRequest allowToolbar(boolean toolbar) {
        this.toolbar = toolbar;
        return this;
    }

    /**
     * @return the background color for the visualization
     */
    public Color getBackground() {
        return background;
    }

    public VisualizationRequest setBackground(Color background) {
        this.background = background;
        return this;
    }

    public VisualizationRequest setPreferredAbstractGraphValidators(List<AbstractGraphValidator> validators){
        this.preferredAbstractGraphValidators = validators;
        return this;
    }

    public VisualizationRequest setPreferredDesignGraphValidators(List<DesignGraphValidator> validators){
        this.preferredDesignGraphValidators = validators;
        return this;
    }

    public List<AbstractGraphValidator> getPreferredAbstractGraphValidators(){
        return this.preferredAbstractGraphValidators;
    }

    public List<DesignGraphValidator> getPreferredDesignGraphValidators(){
        return this.preferredDesignGraphValidators;
    }


    // CONFIGURATION FOR VISUALIZATION
    // modifiable, redesignable ...

    VisualizationRequest(){
        initDefault();
    }

    VisualizationRequest(AbstractGraph graph){
        initDefault();
        this.abstractGraph = graph;
        initAbstractGraph();
    }

    VisualizationRequest(DesignGraph graph){
        initDefault();
        this.designGraph = graph;
        initDesignGraph();
    }

    public AbstractGraph getAbstractGraph(){
        return abstractGraph;
    }

    public DesignGraph getDesignGraph(){
        return designGraph;
    }

    private void initDefault() {
        this.toolbar = true;
        this.allowDifferentDesigner  = true;
        this.allowDifferentLayouter = true;
        this.allowDifferentRouter = true;
        this.movement = true;
        this.pinMovement = false;
        this.allowRedesign = true;
        this.allowDifferentValidators = true;
        this.allowRelayouting = true;
        this.background = null;
        this.allowResetLayout = true;
        this.pins = false;
    }

    private void initAbstractGraph() {
//        throw new UnsupportedOperationException("Not yet implemented");
    }

    private void initDesignGraph() {
//        throw new UnsupportedOperationException("Not yet implemented");
    }
}
