/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api;

import java.awt.Container;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import javax.swing.JPanel;
import org.gpdv.services.validators.AbstractGraphValidator;
import org.gpdv.services.validators.DesignGraphValidator;
import org.gpdv.services.GraphBuilder;
import org.gpdv.services.GraphDesigner;
import org.gpdv.services.GraphLayouter;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.layoutgraph.LayoutGraph;
import org.gpdv.core.api.views.GraphExplorerTopComponent;
import org.gpdv.core.api.views.VisualizationTopComponent;
import org.gpdv.core.api.VisualizationRequest;
import org.gpdv.core.implementation.nodes.GraphNode;
import org.gpdv.services.GraphRouter;
import org.gpdv.core.implementation.scenes.DirectWrapper;
import org.gpdv.core.implementation.scenes.OrthogonalWrapper;
import org.gpdv.core.implementation.scenes.TestLayouter;
import org.gpdv.core.implementation.scenes.TestRouter;
import org.gpdv.core.implementation.scenes.TestRouter2;
import org.gpdv.core.implementation.views.VisualizationManagementPanel;
import org.netbeans.api.visual.widget.ConnectionWidget;
import org.netbeans.api.visual.widget.LayerWidget;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.windows.IOProvider;
import org.openide.windows.OutputWriter;
import org.openide.windows.TopComponent;

/**
 *
 * @author odi
 */
public final class Visualizer {

    private static Visualizer instance;


    private Visualizer(){
    }

    /**
     * Get the usable Visualizer instance
     * @return
     */
    public static Visualizer getVisualizer(){
        if(instance==null) instance = new Visualizer();
        return instance;
    }

    /**
     * Get an unmodified VisualizationRequest
     * @return the unmodified VisualizationRequest
     */
    public VisualizationRequest getVisualizationRequest(){
        return new VisualizationRequest();
    }

    /**
     * Get a VisualizationRequest with AbstractGraph-property set
     * @param graph the AbstractGraph to be visualized
     * @return the VisualizationRequest with set AbstractGraph
     */
    public VisualizationRequest getVisualizationRequest(AbstractGraph graph){
        return new VisualizationRequest(graph);
    }

    /**
     * Get a VisualizationRequest with DesignGraph-property set
     * @param graph the DesignGraph to be visualized
     * @return the VisualizationRequest with set DesignGraph
     */
    public VisualizationRequest getVisualizationRequest(DesignGraph graph){
        return new VisualizationRequest(graph);
    }

    /**
     * Open a new window to display graph passed in request
     * with configuration specified in request
     * @param request
     * @return
     */
    public boolean visualize(VisualizationRequest request){
        OutputWriter io = IOProvider.getDefault().getIO ("Visualizer", false).getOut();
        io.println("----------------");
        io.println("Starting visualization...");
        
        VisualizationTopComponent tc = new VisualizationTopComponent(request);
        io.println("VisualizationTopComponent created...");
        tc.open();
        tc.requestActive();
        return true;
    }


    /**
     * Get a VisualizationRequest used for a standalone-Visualizer
     * @return the VisualizationRequest set to be used as request for a standalone visualization window
     */
    public VisualizationRequest getStandaloneVisualizationRequest() {
        return new VisualizationRequest().standalone(true);
    }

    /**
         * Generates a GraphView with behaviour based on the given VisualizationRequest
         * @param request the VisualizationRequest that determines the behaviour and the contents of the returned GraphView. Don't set AbstractGraph and DesignGraph to visualize data of ExplorerManager.Provider
         * @return the GraphView that visualizes data
         */
        public GraphView getGraphView(VisualizationRequest request){
            return new VisualizationManagementPanel(request);
        }

        /**
         * Generates a GraphView that visualizes the contents of an ExplorerManager.Provider
         * @return the GraphView to visualize the data
         */
        public GraphView getGraphView(){
            return new VisualizationManagementPanel(getSlaveRequest());
        }

        /**
         * Generate a request for 'slave'-mode
         * @return the VisualizationRequest used for the 'slave'-panel
         */
        private static VisualizationRequest getSlaveRequest(){
            return Visualizer.getVisualizer().getVisualizationRequest();
        }

    /**
     * Utilities for the Visualizer
     */
    public static final class Utils{

        private static Lookup.Result<AbstractGraphValidator> availableAbstractGraphValidators;
        private static Lookup.Result<DesignGraphValidator> availableDesignGraphValidators;
        private static Lookup.Result<GraphBuilder> availableGraphBuilders;
        private static Lookup.Result<GraphDesigner> availableGraphDesigners;
        private static Lookup.Result<GraphLayouter> availableGraphLayouters;
        private static Lookup.Result<GraphRouter> availableRouters;

        private static GraphLayouter defaultLayouter;
        private static GraphRouter defaultRouter;

        Utils(){
        }

        /**
         * Toggle the GraphExplorer
         */
        public static void toggleGraphExplorer(){
            GraphExplorerTopComponent tc = GraphExplorerTopComponent.findInstance();
            if(tc.isOpened()) tc.close();
            else tc.open();
        }

        /**
         * Opens a GraphExplorer
         * @return the opened GraphExplorer
         */
        public static TopComponent openGraphExplorer(){
            GraphExplorerTopComponent tc = GraphExplorerTopComponent.findInstance();
            if(!tc.isOpened()){
                tc.open();
                tc.requestActive();
            }
            return tc;
        }

        /**
         * Close the open GraphExplorer
         */
        public static void closeGraphExplorer(){
            GraphExplorerTopComponent tc = GraphExplorerTopComponent.findInstance();
            if(tc.isOpened()) tc.close();
        }

        /**
         * Generate a Node representing the given AbstractGraph
         * @param graph the AbstractGraph the representation is based on
         * @return the Node representing the given AbstractGraph
         */
        public static Node getNodeRepresentation(AbstractGraph graph){
            OutputWriter io = IOProvider.getDefault().getIO ("Visualizer", false).getOut();
            DesignGraph g = DesignGraphFactory.getFactory().convertAbstractToDesignGraph(graph);
            io.println("GraphNode generated");
            return new GraphNode(g);
        }

        /**
         * Generate a Node representing the given DesignGraph
         * @param graph the DesignGraph the representation is based on
         * @return the Node representing the given DesignGraph
         */
        public static Node getNodeRepresentation(DesignGraph graph){
            return new GraphNode(graph);
        }

        /**
         * Get a TopComponent that visualizes AbstractGraphs available in global Lookup
         * @return the Visualization-TopComponent
         */
        public static TopComponent openVisualGraphExplorer(){
            // Default constructor ---> standalone
            return new VisualizationTopComponent();
        }

        /**
         * Get a unmodifiable List of available AbstractGraphValidators
         * @return the unmodifiable List of available AbstractGraphValidators
         */
        public static Collection<AbstractGraphValidator> getAbstractGraphValidators(){
            if(availableAbstractGraphValidators==null){
                availableAbstractGraphValidators = Lookup.getDefault().lookupResult(AbstractGraphValidator.class);
            }
            List<AbstractGraphValidator> l = new ArrayList<AbstractGraphValidator>(availableAbstractGraphValidators.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
         * Get a unmodifiable List of available DesignGraphValidators
         * @return the unmodifiable List of available DesignGraphValidators
         */
        public static Collection<DesignGraphValidator> getDesignGraphValidators(){
            if(availableDesignGraphValidators==null){
                availableDesignGraphValidators = Lookup.getDefault().lookupResult(DesignGraphValidator.class);
            }
            List<DesignGraphValidator> l = new ArrayList<DesignGraphValidator>(availableDesignGraphValidators.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
         * Get a unmodifiable List of available GraphBuilders
         * @return the unmodifiable List of available GraphBuilders
         */
        public static Collection<GraphBuilder> getGraphBuilders(){
            if(availableGraphBuilders==null){
                availableGraphBuilders = Lookup.getDefault().lookupResult(GraphBuilder.class);
            }
            List<GraphBuilder> l = new ArrayList<GraphBuilder>(availableGraphBuilders.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
         * Get a unmodifiable List of available GraphBuilders
         * @return the unmodifiable List of available GraphBuilders
         */
        public static Collection<GraphBuilder> getGraphBuildersForMimeType(String mime){
            if(availableGraphBuilders==null){
                availableGraphBuilders = Lookup.getDefault().lookupResult(GraphBuilder.class);
            }
            List<GraphBuilder> l = new ArrayList<GraphBuilder>(availableGraphBuilders.allInstances());
            List<GraphBuilder> l1 = new ArrayList<GraphBuilder>();
            for(GraphBuilder b : l){
                if(b.getMimeTypes().contains(mime)) l1.add(b);
            }
            return Collections.unmodifiableCollection(l1);
        }

        /**
         * Get a unmodifiable List of available GraphBuilders
         * @return the unmodifiable List of available GraphBuilders
         */
        public static Collection<GraphDesigner> getGraphDesignersForMimeType(String mime){
            if(availableGraphDesigners==null){
                availableGraphDesigners = Lookup.getDefault().lookupResult(GraphDesigner.class);
            }
            List<GraphDesigner> l = new ArrayList<GraphDesigner>(availableGraphDesigners.allInstances());
            List<GraphDesigner> l1 = new ArrayList<GraphDesigner>();
            for(GraphDesigner b : l){
                if(b.getMimeTypes().contains(mime)){
                    l1.add(b);
                } else if(b.getMimeTypes().contains("universal")){
                    l1.add(b);
                }
            }
            return Collections.unmodifiableCollection(l1);
        }

        /**
         * Get a unmodifiable List of available GraphDesigners
         * @return the unmodifiable List of available GraphDesigners
         */
        public static Collection<GraphDesigner> getGraphDesigners(){
            if(availableGraphDesigners==null){
                availableGraphDesigners = Lookup.getDefault().lookupResult(GraphDesigner.class);
            }
            List<GraphDesigner> l = new ArrayList<GraphDesigner>(availableGraphDesigners.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
         * Get a unmodifiable List of available GraphLayouters
         * @return the unmodifiable List of available GraphLayouters
         */
        public static Collection<GraphLayouter> getGraphLayouters(){
            if(availableGraphLayouters==null){
                availableGraphLayouters = Lookup.getDefault().lookupResult(GraphLayouter.class);
            }
            List<GraphLayouter> l = new ArrayList<GraphLayouter>();
            l.add(getDefaultLayouter());
            l.addAll(availableGraphLayouters.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
         * Get a unmodifiable List of available Routers
         * @return the unmodifiable List of available Routers
         */
        public static Collection<GraphRouter> getRouters(){
            if(availableRouters==null){
                availableRouters = Lookup.getDefault().lookupResult(GraphRouter.class);
            }
            List<GraphRouter> l = new ArrayList<GraphRouter>();
            l.add(getDefaultRouter());
            l.add(new DirectWrapper());
            l.add(new TestRouter());
            l.add(new TestRouter2());
            l.add(new OrthogonalWrapper());
            l.addAll(availableRouters.allInstances());
            return Collections.unmodifiableCollection(l);
        }

        /**
     * Get the default GraphLayouter
     * @return the default GraphLayouter
     */
    public static GraphLayouter getDefaultLayouter(){
        if(defaultLayouter==null) defaultLayouter = new DefaultLayouter();
        return defaultLayouter;
    }

    public static GraphRouter getDefaultRouter(){
        if(defaultRouter==null) defaultRouter = new DefaultRouter();
        return defaultRouter;
    }
    }

    /**
     * Dummy Implementation of GraphLayouter
     */
    private static class DefaultLayouter implements GraphLayouter {

        private final String NAME = "DEFAULTGRAPHLAYOUTER";
        private final String DISPLAYNAME = "Default Graph Layouter";
        private final String DESCRIPTION = "Default Graph Layouter implementation included in GPDV bundle";

        public Node getRepresentation() {
            org.openide.nodes.AbstractNode n = new org.openide.nodes.AbstractNode(Children.LEAF);
            n.setName(NAME);
            n.setDisplayName(DISPLAYNAME);
            n.setShortDescription(DESCRIPTION);
            return n;
        }

        public boolean layoutGraph(LayoutGraph graph) {
            return false;
        }

        public boolean isSlotAware() {
            return false;
        }

        @Override
        public String toString() {
            return this.DISPLAYNAME;
        }
    }

    private static class DefaultRouter implements GraphRouter {

        private final String NAME = "DEFAULTROUTER";
        private final String DISPLAYNAME = "Default router";
        private final String DESCRIPTION = "Default router implementation included in GPDV bundle";

        public Node getRepresentation() {
            org.openide.nodes.AbstractNode n = new org.openide.nodes.AbstractNode(Children.LEAF);
            n.setName(NAME);
            n.setDisplayName(DISPLAYNAME);
            n.setShortDescription(DESCRIPTION);
            return n;
        }

        public void setLayerWidget(LayerWidget widget) {
        }

        public List<Point> routeConnection(ConnectionWidget widget) {
            return Collections.emptyList();
        }

    }
}
