/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.designgraph;

import java.awt.Color;
import java.awt.Image;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;


/**
 *
 * @author odi
 */
public class Design {

    public static final int BACKGROUND_POSITION =           0;
    public static final int BACKGROUND_SELECTED_POSITION =  1;
    public static final int BACKGROUND_HOVERED_POSITION =   2;
    public static final int BORDER_POSITION =               3;
    public static final int BORDER_SELECTED_POSITION =      4;
    public static final int BORDER_HOVERED_POSITION =       5;
    public static final int FONT_POSITION =                 6;
    public static final int FONT_SELECTED_POSITION =        7;
    public static final int FONT_HOVERED_POSITION =         8;
    public static final int INCREASED_BACKGROUND_POSITION = 9;
    
    public static final Color DEFAULT_BORDER =                  Color.BLACK;
    public static final Color DEFAULT_BORDER_SELECTED =         Color.BLACK;
    public static final Color DEFAULT_BORDER_HOVERED =          Color.BLACK;
    public static final Color DEFAULT_BACKGROUND =              Color.YELLOW;
    public static final Color DEFAULT_BACKGROUND_SELECTED =     Color.RED;
    public static final Color DEFAULT_BACKGROUND_HOVERED =      Color.BLUE;
    public static final Color DEFAULT_FONT =                    Color.BLACK;
    public static final Color DEFAULT_FONT_SELECTED =           Color.BLACK;
    public static final Color DEFAULT_FONT_HOVERED =            Color.BLACK;
    public static final Color DEFAULT_INCREASED_BACKGROUND =    Color.LIGHT_GRAY;

    public static final int SELECTIONSIZEINCREASE_HORIZONTAL_POSITION = 0;
    public static final int SELECTIONSIZEINCREASE_VERTICAL_POSITION = 1;
    public static final int ARCWIDTH_POSITION = 2;
    public static final int ARCHEIGHT_POSITION = 3;
    public static final int ARCWIDTH_MINIMIZED_POSITION = 4;
    public static final int ARCHEIGHT_MINIMIZED_POSITION = 5;
    public static final int MINIMIZEDWIDTH_POSITION = 6;
    public static final int MINIMIZEDHEIGHT_POSITION = 7;

    public static final int DEFAULT_SELECTIONSIZEINCREASE_HORIZONTAL = 10;
    public static final int DEFAULT_SELECTIONSIZEINCREASE_VERTICAL = 10;
    public static final int DEFAULT_ARCWIDTH = 0;
    public static final int DEFAULT_ARCHEIGHT = 0;
    public static final int DEFAULT_ARCWIDTH_MINIMIZED = 0;
    public static final int DEFAULT_ARCHEIGHT_MINIMIZED = 0;
    public static final int DEFAULT_MINIMIZEDWIDTH = 10;
    public static final int DEFAULT_MINIMIZEDHEIGHT = 10;

    private static final int COLORCOUNT = 10;
    private static final int VALUECOUNT = 8;

    private static Color[] defaultColors;
    private static int[] defaultValues;

    static {
        defaultColors = new Color[COLORCOUNT];
        defaultColors[0] = DEFAULT_BACKGROUND;
        defaultColors[1] = DEFAULT_BACKGROUND_SELECTED;
        defaultColors[2] = DEFAULT_BACKGROUND_HOVERED;
        defaultColors[3] = DEFAULT_BORDER;
        defaultColors[4] = DEFAULT_BORDER_SELECTED;
        defaultColors[5] = DEFAULT_BORDER_HOVERED;
        defaultColors[6] = DEFAULT_FONT;
        defaultColors[7] = DEFAULT_FONT_SELECTED;
        defaultColors[8] = DEFAULT_FONT_HOVERED;
        defaultColors[9] = DEFAULT_INCREASED_BACKGROUND;

        defaultValues = new int[VALUECOUNT];
        defaultValues[0] = DEFAULT_SELECTIONSIZEINCREASE_HORIZONTAL;
        defaultValues[1] = DEFAULT_SELECTIONSIZEINCREASE_VERTICAL;
        defaultValues[2] = DEFAULT_ARCWIDTH;
        defaultValues[3] = DEFAULT_ARCHEIGHT;
        defaultValues[4] = DEFAULT_ARCWIDTH_MINIMIZED;
        defaultValues[5] = DEFAULT_ARCHEIGHT_MINIMIZED;
        defaultValues[6] = DEFAULT_MINIMIZEDWIDTH;
        defaultValues[7] = DEFAULT_MINIMIZEDHEIGHT;
    }

    private Color[] colors;
    private int[] values;
    private Image image;
    private int height;
    private int width;

    public Design(int height, int width){
        this(null, height, width, null);
    }

    public Design(int height, int width, Image image){
        this(null, height, width, image);
    }

    // CONSTRUCTOR CURRENTLY USED BY IMPLEMENTATION
    public Design(Color defaultColor, Color selectedColor, int height, int width){
        this(new Color[]{defaultColor, selectedColor}, height, width, null);
    }

    public Design(Color defaultColor, Color selectedColor, int height, int width, Image image){
        this(new Color[]{defaultColor, selectedColor}, height, width, image);
    }

    public Design(Color[] colors, int height, int width){
        this(colors, height, width, null);
    }

    // CONSTRUCTOR ALWAYS USED
    public Design(Color[] colors, int height, int width, Image image){
        this(colors, defaultValues, height, width, image);
    }

    public Design(Color[] colors, int[] values, int height, int width, Image image){
        this.colors = Arrays.copyOf(defaultColors, COLORCOUNT);
        this.values = Arrays.copyOf(defaultValues, VALUECOUNT);
        if(colors!=null){
            for(int i = 0; i < Math.min(colors.length, COLORCOUNT); i++){
                if(colors[i]!=null){
                    this.colors[i] = colors[i];
                }
            }
        }
        if(values!=null){
            for(int i = 0; i < Math.min(values.length, VALUECOUNT); i++){
                this.values[i] = values[i];
            }
        }
        this.height = height;
        this.width = width;
        this.image = image;
    }

    public static int[] getDefaultValues(){
        return Arrays.copyOf(defaultValues, VALUECOUNT);
    }

    public Color getDefaultColor(){
        return colors[BACKGROUND_POSITION];
    }

    public Color getSelectedColor(){
        return colors[BACKGROUND_SELECTED_POSITION];
    }

    public Color getDefaultBorderColor() {
        return colors[BORDER_POSITION];
    }

    public Color getHoveredBorderColor() {
        return colors[BORDER_HOVERED_POSITION];
    }

    public Color getHoveredColor() {
        return colors[BACKGROUND_HOVERED_POSITION];
    }

    public Image getImage() {
        return image;
    }

    public Color getSelectedBorderColor() {
        return colors[BORDER_SELECTED_POSITION];
    }

    public Color getDefaultFontColor() {
        return colors[FONT_POSITION];
    }

    public Color getHoveredFontColor() {
        return colors[FONT_HOVERED_POSITION];
    }

    public Color getSelectedFontColor() {
        return colors[FONT_SELECTED_POSITION];
    }

    public int getHeight(){
        return height;
    }

    public int getWidth(){
        return width;
    }

    public int getSelectionSizeIncreaseHorizontal(){
        return values[SELECTIONSIZEINCREASE_HORIZONTAL_POSITION];
    }

    public int getSelectionSizeInncreaseVertical(){
        return values[SELECTIONSIZEINCREASE_VERTICAL_POSITION];
    }

    public int getArcWidth(){
        return values[ARCWIDTH_POSITION];
    }

    public int getArcHeight(){
        return values[ARCHEIGHT_POSITION];
    }

    public Color getIncreasedBackgroundColor(){
        return colors[INCREASED_BACKGROUND_POSITION];
    }

    public int getArcHeightMinimized(){
        return values[ARCHEIGHT_MINIMIZED_POSITION];
    }

    public int getArcWidthMinimized(){
        return values[ARCWIDTH_MINIMIZED_POSITION];
    }

    public int getMinimizedWidth(){
        return values[MINIMIZEDWIDTH_POSITION];
    }

    public int getMinimizedHeight(){
        return values[MINIMIZEDHEIGHT_POSITION];
    }
}
