/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.designgraph;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdgeChangeEvent;
import org.gpdv.core.api.abstractgraph.AbstractEdgeChangeListener;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdge;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdgeChangeEvent;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdgeChangeListener;
import org.gpdv.core.api.layoutgraph.LayoutGraph;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlot;

/**
 *
 * @author odi
 */
public class DesignGraphEdge implements LayoutGraphEdge {

    private final long id;
    private final String name;
    private final DesignGraph graph;
    private final AbstractGraphEdge abstractEdge;
    private DesignGraphSlot sourceSlot;
    private DesignGraphSlot destinationSlot;
    private Design design;
    private final List<DesignGraphEdgeChangeListener> designListener;
    private final List<LayoutGraphEdgeChangeListener> layoutListener;
    private final List<Point> controlPoints;

    private boolean inserted;

    public DesignGraphEdge(final long edgeID, final AbstractGraphEdge edge, final DesignGraph graph, final DesignGraphSlot source, final DesignGraphSlot destination, final Design design) throws IllegalArgumentException{
        if(graph==null||edge==null||source==null||destination==null||design==null){
            throw new IllegalArgumentException();
        }
        if(!source.getDesignNode().getOutgoingSlots().contains(source)) throw new IllegalArgumentException();
        if(!destination.getDesignNode().getIncomingSlots().contains(destination)) throw new IllegalArgumentException();
        this.id = edgeID;
        designListener = new ArrayList<DesignGraphEdgeChangeListener>();
        layoutListener = new ArrayList<LayoutGraphEdgeChangeListener>();
        controlPoints = new ArrayList<Point>();
        this.abstractEdge = edge;
        this.name = edge.getName();
        this.graph = graph;
        this.design = design;
        this.sourceSlot = source;
        this.destinationSlot = destination;
        this.inserted = false;
        if(source.getDesignNode().getDesignGraph()!=destination.getDesignNode().getDesignGraph()||source.getDesignNode().getDesignGraph()!=graph||!graph.getDesignNodes().contains(source.getDesignNode())||!graph.getDesignNodes().contains(destination.getDesignNode())||!graph.addDesignEdge(this)){
            throw new IllegalArgumentException();
        }
//        edge.addAbstractEdgeChangeListener(this);
    }

    final void inserted(){
        this.inserted = true;
    }

    final void removed(){
        this.inserted = false;
    }

    final boolean isInserted(){
        return inserted;
    }

    public final DesignGraph getDesignGraph(){
        return graph;
    }

    public final AbstractGraphEdge getAbstractEdge(){
        return abstractEdge;
    }

    public final DesignGraphSlot getSourceSlot(){
        return sourceSlot;
    }

    public final DesignGraphSlot getDestinationSlot(){
        return destinationSlot;
    }

    public final boolean setSource(final DesignGraphSlot source){
        if(source==null||source==this.sourceSlot||source.getDesignNode().getDesignGraph()!=this.graph||!source.getDesignNode().isInserted()) return false;
        beforeSetSource(source);
        DesignGraphEdgeChangeEvent e = new DesignGraphEdgeChangeEvent(this, DesignGraphEdgeChangeEvent.NEW_SOURCE, this.sourceSlot, null, null);
        LayoutGraphEdgeChangeEvent e1 = new LayoutGraphEdgeChangeEvent(this, LayoutGraphEdgeChangeEvent.NEW_SOURCE, this.sourceSlot, null);
        if(inserted) this.sourceSlot.removeDesignEdge(this);
        this.sourceSlot = source;
        if(inserted) this.sourceSlot.addDesignEdge(this);
        this.fireDesignEdgeChangeEvent(e);
        this.fireLayoutEdgeChangeEvent(e1);
        afterSetSource(source);
        return true;
    }

    public final boolean setDestination(final DesignGraphSlot destination){
        if(destination==null||destination==this.sourceSlot||destination.getDesignNode().getDesignGraph()!=this.graph||!destination.getDesignNode().isInserted()) return false;
        beforeSetDestination(destination);
        DesignGraphEdgeChangeEvent e = new DesignGraphEdgeChangeEvent(this, DesignGraphEdgeChangeEvent.NEW_DESTINATION, this.destinationSlot, null, null);
        LayoutGraphEdgeChangeEvent e1 = new LayoutGraphEdgeChangeEvent(this, LayoutGraphEdgeChangeEvent.NEW_DESTINATION, this.destinationSlot, null);
        if(inserted) this.destinationSlot.removeDesignEdge(this);
        this.destinationSlot = destination;
        if(inserted) this.destinationSlot.addDesignEdge(this);
        this.fireDesignEdgeChangeEvent(e);
        this.fireLayoutEdgeChangeEvent(e1);
        afterSetDestination(destination);
        return true;
    }

    public synchronized final void addDesignGraphEdgeChangeListener(final DesignGraphEdgeChangeListener listener){
        this.designListener.add(listener);
    }

    public synchronized final void removeDesignGraphEdgeChangeListener(final DesignGraphEdgeChangeListener listener){
        this.designListener.remove(listener);
    }

    public final Design getDesign(){
        return design;
    }

    public final boolean setDesign(final Design design){
        if(design==null||design==this.design) return false;
        beforeSetDesign(design);
        DesignGraphEdgeChangeEvent e = new DesignGraphEdgeChangeEvent(this, DesignGraphEdgeChangeEvent.NEW_DESIGN, null, this.design, null);
        this.design = design;
        this.fireDesignEdgeChangeEvent(e);
        afterSetDesign(design);
        return true;
    }

    public final LayoutGraph getLayoutGraph() {
        return this.graph;
    }

    public final String getName() {
        return name;
    }

    public final long getID() {
        return id;
    }

    public final DesignGraphEdge getDesignEdge() {
        return this;
    }

    public final LayoutGraphSlot getSourceLayoutSlot() {
        return sourceSlot;
    }

    public final LayoutGraphSlot getDestinationLayoutSlot() {
        return destinationSlot;
    }

    public final boolean addControlPoint(final Point p) {
        if(p==null) return false;
        beforeAddControlPoint(p);
        LayoutGraphEdgeChangeEvent e = new LayoutGraphEdgeChangeEvent(this, LayoutGraphEdgeChangeEvent.CONTROL_POINT_ADDED, null, p);
        this.controlPoints.add(p);
        this.fireLayoutEdgeChangeEvent(e);
        afterAddControlPoint(p);
        return true;
    }

    public final boolean removeControlPoint(final Point p) {
        if(p==null||!this.controlPoints.contains(p)) return false;
        beforeRemoveControlPoint(p);
        LayoutGraphEdgeChangeEvent e = new LayoutGraphEdgeChangeEvent(this, LayoutGraphEdgeChangeEvent.CONTROL_POINT_REMOVED, null, p);
        this.controlPoints.remove(p);
        this.fireLayoutEdgeChangeEvent(e);
        afterRemoveControlPoint(p);
        return true;
    }

    public final boolean removeControlPointByCoords(final Point p) {
        if(p==null) return false;
        for(Point o : getControlPoints()){
            if(o.getX()==p.getX()&&o.getY()==p.getY())
                beforeRemoveControlPointByCoords(o);
                LayoutGraphEdgeChangeEvent e = new LayoutGraphEdgeChangeEvent(this, LayoutGraphEdgeChangeEvent.CONTROL_POINT_REMOVED, null, o);
                controlPoints.remove(o);
                this.fireLayoutEdgeChangeEvent(e);
                afterRemoveControlPointByCoords(o);
                return true;
        }
        return false;
    }

    public final List<Point> getControlPoints() {
        List<Point> l = new ArrayList<Point>();
        l.addAll(controlPoints);
        return Collections.unmodifiableList(l);
    }

    public final void addLayoutGraphEdgeChangeListener(final LayoutGraphEdgeChangeListener listener) {
        this.layoutListener.add(listener);
    }

    public final void removeLayoutGraphEdgeChangeListener(final LayoutGraphEdgeChangeListener listener) {
        this.layoutListener.remove(listener);
    }

    private synchronized final void fireDesignEdgeChangeEvent(final DesignGraphEdgeChangeEvent e) {
        if(!inserted) return;
        for(DesignGraphEdgeChangeListener l : designListener){
            l.designEdgeChangePerformed(e);
        }
    }

    private synchronized final void fireLayoutEdgeChangeEvent(final LayoutGraphEdgeChangeEvent e){
        if(!inserted) return;
        for(LayoutGraphEdgeChangeListener l : layoutListener){
            l.layoutEdgeChangePerformed(e);
        }
    }

    protected void beforeSetSource(final DesignGraphSlot source) {
    }

    protected void afterSetSource(final DesignGraphSlot source) {
    }

    protected void beforeSetDestination(final DesignGraphSlot destination) {
    }

    protected void afterSetDestination(final DesignGraphSlot destination) {
    }

    protected void beforeSetDesign(final Design design) {
    }

    protected void afterSetDesign(final Design design) {
    }

    protected void beforeAddControlPoint(final Point p) {
    }

    protected void afterAddControlPoint(final Point p) {
    }

    protected void beforeRemoveControlPoint(final Point p) {
    }

    protected void afterRemoveControlPoint(final Point p) {
    }

    protected void beforeRemoveControlPointByCoords(final Point p) {
    }

    protected void afterRemoveControlPointByCoords(final Point p) {
    }
}
