/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.designgraph;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.layoutgraph.LayoutGraph;
import org.gpdv.core.api.layoutgraph.LayoutGraphNode;
import org.gpdv.core.api.layoutgraph.LayoutGraphNodeChangeEvent;
import org.gpdv.core.api.layoutgraph.LayoutGraphNodeChangeListener;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlot;

/**
 *
 * @author odi
 */
public class DesignGraphNode implements LayoutGraphNode{

    private final DesignGraph graph;
    private final AbstractGraphNode node;
    private final long id;
    private final String name;
    private final List<DesignGraphSlot> incomingSlots;
    private final List<DesignGraphSlot> outgoingSlots;
    private Design design;
    private final List<DesignGraphNodeChangeListener> designListener;
    private final List<LayoutGraphNodeChangeListener> layoutListener;
    private Point point;

    private boolean inserted;

    public DesignGraphNode(long nodeID, AbstractGraphNode node, DesignGraph graph, Design design) throws IllegalArgumentException{
        if(node==null||graph==null||design==null||node.getGraph()!=graph.getAbstractGraph()) throw new IllegalArgumentException();
        this.id = nodeID;
        incomingSlots = new ArrayList<DesignGraphSlot>();
        outgoingSlots = new ArrayList<DesignGraphSlot>();
        designListener = new ArrayList<DesignGraphNodeChangeListener>();
        layoutListener = new ArrayList<LayoutGraphNodeChangeListener>();
        this.node = node;
        this.name = node.getName();
        this.graph = graph;
        this.design = design;
        this.point = new Point(-1,-1);
        this.inserted = false;
        if(!graph.addDesignNode(this)) throw new IllegalArgumentException();
    }

    final void inserted(){
        this.inserted = true;
    }

    final void removed(){
        this.inserted = false;
    }
    
    public final DesignGraph getDesignGraph(){
        return graph;
    }

    public final AbstractGraphNode getAbstractNode(){
        return node;
    }

    public final List<DesignGraphSlot> getIncomingSlots(){
        List<DesignGraphSlot> l = new ArrayList<DesignGraphSlot>();
        l.addAll(incomingSlots);
        return Collections.unmodifiableList(l);
    }

    public final List<DesignGraphSlot> getOutgoingSlots(){
        List<DesignGraphSlot> l = new ArrayList<DesignGraphSlot>();
        l.addAll(outgoingSlots);
        return Collections.unmodifiableList(l);
    }

    public final boolean addIncomingSlot(final DesignGraphSlot slot){
        if(slot==null||slot.getDesignNode()!=this||incomingSlots.contains(slot)||outgoingSlots.contains(slot)) return false;
        beforeAddIncomingSlot(slot);
        DesignGraphNodeChangeEvent e = new DesignGraphNodeChangeEvent(this, DesignGraphNodeChangeEvent.INCOMING_SLOT_ADDED, slot, null, null);
        this.incomingSlots.add(slot);
        this.fireDesignNodeChangeEvent(e);
        afterAddIncomingSlot(slot);
        return true;
    }
    public final boolean addOutgoingSlot(final DesignGraphSlot slot){
        if(slot==null||slot.getDesignNode()!=this||incomingSlots.contains(slot)||outgoingSlots.contains(slot)) return false;
        beforeAddOutgoingSlot(slot);
        DesignGraphNodeChangeEvent e = new DesignGraphNodeChangeEvent(this, DesignGraphNodeChangeEvent.OUTGOING_SLOT_ADDED, slot, null, null);
        this.outgoingSlots.add(slot);
        this.fireDesignNodeChangeEvent(e);
        afterAddOutoingSlot(slot);
        return true;
    }
    public final boolean removeIncomingSlot(final DesignGraphSlot slot){
        if(slot==null||slot.getDesignNode()!=this||!incomingSlots.contains(slot)) return false;
        beforeRemoveIncomingSlot(slot);
        DesignGraphNodeChangeEvent e = new DesignGraphNodeChangeEvent(this, DesignGraphNodeChangeEvent.INCOMING_SLOT_REMOVED, slot, null, null);
        this.incomingSlots.remove(slot);
        this.fireDesignNodeChangeEvent(e);
        afterRemoveIncomingSlot(slot);
        return true;
    }
    public final boolean removeOutgoingSlot(final DesignGraphSlot slot){
        if(slot==null||slot.getDesignNode()!=this||!outgoingSlots.contains(slot)) return false;
        beforeRemoveOutgoingSlot(slot);
        DesignGraphNodeChangeEvent e = new DesignGraphNodeChangeEvent(this, DesignGraphNodeChangeEvent.OUTGOING_SLOT_REMOVED, slot, null, null);
        this.outgoingSlots.remove(slot);
        this.fireDesignNodeChangeEvent(e);
        afterRemoveOutgoingSlot(slot);
        return true;
    }
    public synchronized final void addDesignGraphNodeChangeListener(DesignGraphNodeChangeListener listener){
        this.designListener.add(listener);
    }
    public synchronized final void removeDesignGraphNodeChangeListener(DesignGraphNodeChangeListener listener){
        this.designListener.remove(listener);
    }
    public final Design getDesign(){
        return design;
    }
    public final boolean setDesign(final Design design){
        if(design==null||design==this.design) return false;
        beforeSetDesign(design);
        DesignGraphNodeChangeEvent e = new DesignGraphNodeChangeEvent(this, DesignGraphEdgeChangeEvent.NEW_DESIGN, null, this.design, null);
        this.design = design;
        this.fireDesignNodeChangeEvent(e);
        afterSetDesign(design);
        return true;
    }
    final boolean isInserted(){
        return inserted;
    }

    public final LayoutGraph getLayoutGraph() {
        return this.graph;
    }

    public final String getName() {
        return name;
    }

    public final long getID() {
        return id;
    }

    public final DesignGraphNode getDesignNode() {
        return this;
    }

    public final List<LayoutGraphSlot> getIncomingLayoutSlots() {
        List<LayoutGraphSlot> l = new ArrayList<LayoutGraphSlot>();
        l.addAll(incomingSlots);
        return Collections.unmodifiableList(l);
    }

    public final List<LayoutGraphSlot> getOutgoingLayoutSlots() {
        List<LayoutGraphSlot> l = new ArrayList<LayoutGraphSlot>();
        l.addAll(outgoingSlots);
        return Collections.unmodifiableList(l);
    }

    public final Point getPoint() {
        return point;
    }

    public final boolean setPoint(final Point p) {
        if(p==null||p==point) return false;
        beforeSetPoint(p);
        LayoutGraphNodeChangeEvent e = new LayoutGraphNodeChangeEvent(this, LayoutGraphNodeChangeEvent.NEW_POINT, null, point);
        this.point = p;
        fireLayoutNodeChangeEvent(e);
        afterSetPoint(p);
        return true;
    }

    public final int getWidth() {
        return design.getWidth();
    }

    public final int getHeight() {
        return design.getHeight();
    }

    public synchronized final void addLayoutGraphNodeChangeListener(final LayoutGraphNodeChangeListener listener) {
        this.layoutListener.add(listener);
    }

    public synchronized final void removeLayoutGraphNodeChangeListener(final LayoutGraphNodeChangeListener listener) {
        this.layoutListener.remove(listener);
    }

    private synchronized final void fireDesignNodeChangeEvent(final DesignGraphNodeChangeEvent e) {
        if(!inserted) return;
        for(DesignGraphNodeChangeListener l : designListener){
            l.designNodeChangePerformed(e);
        }
    }

    private synchronized final void fireLayoutNodeChangeEvent(final LayoutGraphNodeChangeEvent e) {
        if(!inserted) return;
        for(LayoutGraphNodeChangeListener l : layoutListener){
            l.layoutNodeChangePerformed(e);
        }
    }

    protected void beforeAddIncomingSlot(final DesignGraphSlot slot) {
    }

    protected void afterAddIncomingSlot(final DesignGraphSlot slot) {
    }

    protected void beforeAddOutgoingSlot(final DesignGraphSlot slot) {
    }

    protected void afterAddOutoingSlot(final DesignGraphSlot slot) {
    }

    protected void beforeRemoveIncomingSlot(final DesignGraphSlot slot) {
    }

    protected void afterRemoveIncomingSlot(final DesignGraphSlot slot) {
    }

    protected void beforeRemoveOutgoingSlot(final DesignGraphSlot slot) {
    }

    protected void afterRemoveOutgoingSlot(final DesignGraphSlot slot) {
    }

    protected void beforeSetDesign(final Design design) {
    }

    protected void afterSetDesign(final Design design) {
    }

    protected void beforeSetPoint(final Point p) {
    }

    protected void afterSetPoint(final Point p) {
    }
}
