/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.designgraph;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdge;
import org.gpdv.core.api.layoutgraph.LayoutGraphNode;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlot;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlotChangeEvent;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlotChangeListener;

/**
 *
 * @author odi
 */
public class DesignGraphSlot implements LayoutGraphSlot{

    public static final String INCOMING = "INCOMINGSLOT";
    public static final String OUTGOING = "OUTGOINGSLOT";

    private final DesignGraphNode node;
    private final List<DesignGraphEdge> edges;
    private Design design;
    private final List<DesignGraphSlotChangeListener> slotListener;
    private final List<LayoutGraphSlotChangeListener> layoutSlotListener;
    private Point point;

    public DesignGraphSlot(DesignGraphNode node, String mode, Design design) throws IllegalArgumentException{
        if(node==null||mode==null||design==null) throw new IllegalArgumentException();
        this.node = node;
        edges = new ArrayList<DesignGraphEdge>();
        slotListener = new ArrayList<DesignGraphSlotChangeListener>();
        layoutSlotListener = new ArrayList<LayoutGraphSlotChangeListener>();
        this.point = new Point(-1,-1);
        this.design = design;
        boolean b = false;
        if(mode.equals(INCOMING)){
            b = node.addIncomingSlot(this);
        } else if(mode.equals(OUTGOING)){
            b = node.addOutgoingSlot(this);
        }
        // if slot could not be inserted throw exception
        if(!b) throw new IllegalArgumentException();
    }

    public DesignGraphNode getDesignNode(){
        return node;
    }

    public final List<DesignGraphEdge> getDesignEdges(){
        List<DesignGraphEdge> l = new ArrayList<DesignGraphEdge>();
        l.addAll(edges);
        return Collections.unmodifiableList(l);
    }

    public final boolean addDesignEdge(final DesignGraphEdge edge){
        if(edge==null||edges.contains(edge)||edge.getDesignGraph()!=node.getDesignGraph()) return false;
        beforeAddDesignEdge(edge);
        DesignGraphSlotChangeEvent e = new DesignGraphSlotChangeEvent(this, DesignGraphSlotChangeEvent.EDGE_ADDED, edge, null);
        LayoutGraphSlotChangeEvent e1 = new LayoutGraphSlotChangeEvent(this, LayoutGraphSlotChangeEvent.EDGE_ADDED, edge, null);
        edges.add(edge);
        this.fireSlotChangeEvent(e);
        this.fireLayoutSlotChangeEvent(e1);
        afterAddDesignEdge(edge);
        return true;
    }

    public final boolean removeDesignEdge(final DesignGraphEdge edge){
        if(edge==null||!edges.contains(edge)||edge.getDesignGraph()!=node.getDesignGraph()) return false;
        beforeRemoveDesignEdge(edge);
        DesignGraphSlotChangeEvent e = new DesignGraphSlotChangeEvent(this, DesignGraphSlotChangeEvent.EDGE_REMOVED, edge, null);
        LayoutGraphSlotChangeEvent e1 = new LayoutGraphSlotChangeEvent(this, LayoutGraphSlotChangeEvent.EDGE_REMOVED, edge, null);
        edges.remove(edge);
        this.fireSlotChangeEvent(e);
        this.fireLayoutSlotChangeEvent(e1);
        afterRemoveDesignEdge(edge);
        return true;
    }

    public final Design getDesign(){
        return design;
    }

    public final boolean setDesign(final Design design){
        if(design==null||design==this.design) return false;
        beforeSetDesign(design);
        DesignGraphSlotChangeEvent e = new DesignGraphSlotChangeEvent(this, DesignGraphSlotChangeEvent.NEW_DESIGN, null, this.design);
        this.design = design;
        this.fireSlotChangeEvent(e);
        afterSetDesign(design);
        return false;
    }

    public synchronized final void addDesignGraphSlotChangeListener(final DesignGraphSlotChangeListener listener){
        this.slotListener.add(listener);
    }
    public synchronized final void removeDesignGraphSlotChangeListener(final DesignGraphSlotChangeListener listener){
        this.slotListener.remove(listener);
    }

    public final LayoutGraphNode getLayoutNode() {
        return node;
    }

    public final List<LayoutGraphEdge> getLayoutEdges() {
        List<LayoutGraphEdge> l = new ArrayList<LayoutGraphEdge>();
        l.addAll(edges);
        return Collections.unmodifiableList(l);
    }

    public final Point getPoint() {
        return point;
    }

    public final boolean setPoint(final Point p) {
        if(p==null||p==point) return false;
        beforeSetPoint(p);
        LayoutGraphSlotChangeEvent e = new LayoutGraphSlotChangeEvent(this, LayoutGraphSlotChangeEvent.NEW_POINT, null, point);
        this.point = p;
        this.fireLayoutSlotChangeEvent(e);
        afterSetPoint(p);
        return true;
    }

    public final int getWidth() {
        return design.getWidth();
    }

    public final int getHeight() {
        return design.getHeight();
    }

    public synchronized final void addLayoutGraphSlotChangeListener(final LayoutGraphSlotChangeListener listener) {
        this.layoutSlotListener.add(listener);
    }

    public synchronized final void removeLayoutGraphSlotChangeListener(final LayoutGraphSlotChangeListener listener) {
        this.layoutSlotListener.remove(listener);
    }

    private synchronized final void fireSlotChangeEvent(final DesignGraphSlotChangeEvent e){
        if(!this.node.isInserted()) return;
        for(DesignGraphSlotChangeListener l : slotListener){
            l.slotChangePerformed(e);
        }
    }

    private synchronized final void fireLayoutSlotChangeEvent(final LayoutGraphSlotChangeEvent e){
        if(!this.node.isInserted()) return;
        for(LayoutGraphSlotChangeListener l : layoutSlotListener){
            l.layoutSlotChangeEventPerformed(e);
        }
    }

    private void beforeAddDesignEdge(final DesignGraphEdge edge) {
    }

    private void afterAddDesignEdge(final DesignGraphEdge edge) {
    }

    private void beforeRemoveDesignEdge(final DesignGraphEdge edge) {
    }

    private void afterRemoveDesignEdge(final DesignGraphEdge edge) {
    }

    private void beforeSetDesign(final Design design) {
    }

    private void afterSetDesign(final Design design) {
    }

    private void beforeSetPoint(final Point p) {
    }

    private void afterSetPoint(final Point p) {
    }
}
