/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.nodes;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.swing.Action;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Sheet;
import org.openide.util.WeakListeners;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author odi
 */
public final class GraphNode extends AbstractNode implements PropertyChangeListener{

    private DesignGraph graph;
    private List<GraphChangeListener> listener;
    public GraphNode(DesignGraph graph){
        super(new GraphChildren(graph), new ProxyLookup(Lookups.fixed(new Object[]{graph, graph.getAbstractGraph()}), graph.getAbstractGraph().getLookup()));
        this.setName(graph.getName());
        this.graph = graph;
        graph.getAbstractGraph().addPropertyChangeListener(WeakListeners.propertyChange(this, graph.getAbstractGraph()));
        this.listener = new ArrayList<GraphChangeListener>();
    }

    public DesignGraph getGraph(){
        return graph;
    }

    @Override
    public Action[] getActions(boolean context) {
        return ActionProvider.getActions("Graph", this);
    }

    public void fireGraphChangeEvent(GraphChangeEvent e){
        for(GraphChangeListener l : listener){
            l.graphChangePerformed(e);
        }
    }

    public void addGraphChangeListener(GraphChangeListener l){
        this.listener.add(l);
    }

    public void removeGraphChangeListener(GraphChangeListener l){
        this.listener.remove(l);
    }

    @Override
    protected Sheet createSheet() {
        Sheet sheet = Sheet.createDefault();
        Sheet.Set set = Sheet.createPropertiesSet();

        Map<String, String> properties = this.getGraph().getAbstractGraph().getProperties();
        for(String prop : properties.keySet()){
            set.put(new PropertyImpl(graph.getAbstractGraph(), prop, properties.get(prop)));
        }
        sheet.put(set);
        return sheet;
    }

    public void propertyChange(PropertyChangeEvent evt) {
        this.setSheet(this.createSheet());
    }

}
