/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.scenes;

import org.gpdv.services.GraphRouter;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.designgraph.DesignGraphNode;
import org.gpdv.core.api.GraphUtils;
import org.gpdv.core.api.VisualizationRequest;
import org.gpdv.core.implementation.nodes.EdgeNode;
import org.gpdv.core.implementation.nodes.NodeNode;
import org.gpdv.core.implementation.widgets.EdgeWidget;
import org.gpdv.core.implementation.widgets.ImageNodeWidget;
import org.gpdv.core.implementation.widgets.LabelNodeWidget;
import org.gpdv.core.implementation.widgets.NodeWidgetInterface;
import org.netbeans.api.visual.action.ActionFactory;
import org.netbeans.api.visual.action.EditProvider;
import org.netbeans.api.visual.action.MoveProvider;
import org.netbeans.api.visual.action.PopupMenuProvider;
import org.netbeans.api.visual.action.SelectProvider;
import org.netbeans.api.visual.action.WidgetAction;
import org.netbeans.api.visual.anchor.Anchor;
import org.netbeans.api.visual.anchor.AnchorFactory;
import org.netbeans.api.visual.anchor.AnchorShape;
import org.netbeans.api.visual.graph.GraphScene;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.router.Router;
import org.netbeans.api.visual.router.RouterFactory;
import org.netbeans.api.visual.widget.ConnectionWidget;
import org.netbeans.api.visual.widget.LayerWidget;
import org.netbeans.api.visual.widget.Widget;
import org.netbeans.api.visual.widget.general.IconNodeWidget;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.actions.Presenter;

/**
 *
 * @author odi
 */
public class GraphSceneImpl extends GraphScene<NodeNode, EdgeNode> implements GraphSceneInterface{


    private LayerWidget mainLayer;
    private LayerWidget connectionLayer;

    private WidgetAction selectAction;
    private WidgetAction hoverAction;
    private WidgetAction popupAction;

    private VisualizationRequest request;

    private Set<Widget> movedWidgets;

    private Router freeRouter;
    private Router directRouter;

    private GraphRouter router;

    private JScrollPane scrollPane;

    private ScenePopupMenuProvider scenePopupMenuProvider;

    private Icon right;
    private ResourceBundle bundle;

    public GraphSceneImpl(VisualizationRequest request, SelectProvider selectProvider, JScrollPane scrollPane){
        super();
        this.request = request;
        this.scrollPane = scrollPane;
        this.freeRouter = RouterFactory.createFreeRouter();
        this.directRouter = RouterFactory.createDirectRouter();
        this.mainLayer = new LayerWidget(this);
        mainLayer.setLayout(LayoutFactory.createAbsoluteLayout());
        if(request.getBackground()!=null){
            mainLayer.setBackground(request.getBackground());
            mainLayer.setOpaque(true);
        }
        this.connectionLayer = new LayerWidget(this);
        this.popupAction = ActionFactory.createPopupMenuAction(new NodePopupMenuProvider());
        this.selectAction = ActionFactory.createSelectAction(selectProvider);
        this.hoverAction = this.createObjectHoverAction();
        this.scenePopupMenuProvider = new ScenePopupMenuProvider(this, scrollPane, selectProvider);
        this.getActions().addAction(selectAction);
        this.getActions().addAction(ActionFactory.createCenteredZoomAction(1.1));
        this.getActions().addAction(ActionFactory.createPanAction());
        this.getActions().addAction(ActionFactory.createPopupMenuAction(scenePopupMenuProvider));
        this.addChild(mainLayer);
        this.addChild(connectionLayer);
        movedWidgets = new HashSet<Widget>();
        this.bundle = NbBundle.getBundle(GraphSceneImpl.class);
        right = ImageUtilities.image2Icon(ImageUtilities.loadImage(bundle.getString("right")));
    }

    @Override
    protected Widget attachNodeWidget(NodeNode node) {
        Widget w;
        if(node.getNode().getDesign().getImage()==null){
            w = new LabelNodeWidget(node, this);
        } else {
            w = new ImageNodeWidget(node, this);
        }
        w.getActions().addAction(popupAction);
        w.getActions().addAction(hoverAction);
        w.getActions().addAction(selectAction);
        if(request.allowMovement()) w.getActions().addAction(ActionFactory.createMoveAction(null, new MoveProviderImpl()));
        w.getActions().addAction(ActionFactory.createEditAction(new EditProviderImpl()));
        mainLayer.addChild(w);
        return w;
    }

    @Override
    protected Widget attachEdgeWidget(EdgeNode edge) {
        ConnectionWidget w = new EdgeWidget(edge, this, mainLayer);
        w.setTargetAnchorShape(AnchorShape.TRIANGLE_FILLED);
        if(!request.allowMovement()) w.setControlPoints(edge.getEdge().getControlPoints(), true);
        if(router==null){
            if(!request.allowMovement()){
                w.setRouter(freeRouter);
            } else {
                w.setRouter(directRouter);
            }
        } else {
            w.setRouter(router);
        }
        w.getActions().addAction(popupAction);
        w.getActions().addAction(hoverAction);
        w.getActions().addAction(selectAction);
        w.getActions().addAction(ActionFactory.createEditAction(new EditProviderImpl()));
        connectionLayer.addChild(w);
        return w;
    }

    @Override
    protected void attachEdgeSourceAnchor(EdgeNode edge, NodeNode oldSourceNode, NodeNode sourceNode) {
        ConnectionWidget c = (ConnectionWidget) findWidget(edge);
        Widget p = findWidget(sourceNode);
        Anchor a = AnchorFactory.createRectangularAnchor(p);
        c.setSourceAnchor(a);
    }

    @Override
    protected void attachEdgeTargetAnchor(EdgeNode edge, NodeNode oldTargetNode, NodeNode targetNode) {
        ConnectionWidget c = (ConnectionWidget) findWidget(edge);
        Widget p = findWidget(targetNode);
        Anchor a = AnchorFactory.createRectangularAnchor(p);
        c.setTargetAnchor(a);
    }

    public ObjectScene getObjectScene() {
        return this;
    }

    public synchronized void newGraph(Node graph) {
        boolean newNodes = false;
        scenePopupMenuProvider.setGraph(graph);
        Set<NodeNode> nodeSet = new HashSet<NodeNode>(this.getNodes());
        Set<EdgeNode> edgeSet = new HashSet<EdgeNode>(this.getEdges());
        Map<DesignGraphNode, NodeNode> nodes = new HashMap<DesignGraphNode, NodeNode>();
        for(Node n : graph.getChildren().getNodes()){                   // NodeListNode and EdgeListNode
            for(Node no : n.getChildren().getNodes()){                  // NodeNode for NodeListNode, EdgeNode for EdgeListNode
                if(no instanceof NodeNode){                             // NodeNode in NodeListNode
                    if(!nodeSet.contains((NodeNode)no)){
                        this.addNode((NodeNode)no);                        // Add NodeNode
                        newNodes = true;
                    } else {
                        nodeSet.remove((NodeNode)no);
                    }
                    nodes.put(((NodeNode)no).getNode(), (NodeNode)no);
                } else if(no instanceof EdgeNode){                      // EdgeNode in EdgeListNode, NodeNodes and SlotNodes already traversed
                    NodeNode source = nodes.get(((EdgeNode)no).getEdge().getSourceSlot().getDesignNode());
                    NodeNode destination = nodes.get(((EdgeNode)no).getEdge().getDestinationSlot().getDesignNode());
                    if(source==null||destination==null){
                        // error
                    } else if(source!=destination){
                        if(!edgeSet.contains((EdgeNode)no)){
                            this.addEdge((EdgeNode)no);
                            this.setEdgeSource(((EdgeNode)no), source);
                            this.setEdgeTarget(((EdgeNode)no), destination);
                        } else {
                            edgeSet.remove((EdgeNode)no);
                        }
                    }
                }
            }
        }
        for(EdgeNode e : edgeSet){
            this.removeEdge(e);
        }
        for(NodeNode n : nodeSet){
            this.removeNodeWithEdges(n);
        }
        if(newNodes){
            for(NodeNode n : this.getNodes()){
                Widget w = findWidget(n);
                if(w!=null){
                    w.setPreferredLocation(n.getNode().getPoint());
                }
            }
        } else {
            for(NodeNode n : this.getNodes()){
                Widget w = this.findWidget(n);
                if(w!=null){
                    n.getNode().setPoint(w.getPreferredLocation());
                }
            }
        }
    }

    public GraphScene getGraphScene() {
        return this;
    }

    public void clear() {
        for(NodeNode n : new ArrayList<NodeNode>(this.getNodes())){
            this.removeNodeWithEdges(n);
        }
        this.revalidate();
    }

    public void enforceLayout(boolean force) {
        if(force){
            for(NodeNode n : this.getNodes()){
                Widget w = this.findWidget(n);
                w.setPreferredLocation(n.getNode().getPoint());
            }
        } else {
            for(Widget w : this.movedWidgets){
                Object o = this.findObject(w);
                if(o instanceof NodeNode){
                    w.setPreferredLocation(((NodeNode)o).getNode().getPoint());
                }
            }
        }
        this.movedWidgets.clear();
    }

//    private void resetControlPoints(){
//        for(EdgeNode n : this.getEdges()){
//            ConnectionWidget w = (ConnectionWidget)this.findWidget(n);
//            w.setControlPoints(n.getEdge().getControlPoints(), true);
//        }
//    }

    public void setFreeRouter(boolean b) {
        if(b){
            for(EdgeNode n : this.getEdges()){
                ConnectionWidget w = (ConnectionWidget) this.findWidget(n);
                List<Point> p = Collections.emptyList();
                w.setControlPoints(p, true);
                w.setRouter(freeRouter);
            }
        } else {
            if(router==null){
                for(EdgeNode n : this.getEdges()){
                    ConnectionWidget w = (ConnectionWidget) this.findWidget(n);
//                    w.setControlPoints(Collections.EMPTY_LIST, true);
                    if(!request.allowMovement()){
                        w.setControlPoints(n.getEdge().getControlPoints(), true);
                        w.setRouter(freeRouter);
                    } else {
                        w.setRouter(directRouter);
                    }
                }
            } else {
                for(EdgeNode n : this.getEdges()){
                    ConnectionWidget w = (ConnectionWidget) this.findWidget(n);
                    w.setRouter(router);
                }
            }
        }
    }

    public void setRouter(GraphRouter router) {
        this.router = router;
        if(router!=null)this.router.setLayerWidget(mainLayer);
    }

    public void focusSelectedObject() {
        Object selected = this.getSelectedObjects().iterator().next();
        if(selected!=null){
            Widget w = this.findWidget(selected);
            if(w!=null){
                scrollToWidget(w);
            }
        }
    }

    public void totallyNewGraph(Node node) {
        for(NodeNode n : new LinkedList<NodeNode>(this.getNodes())){
            this.removeNodeWithEdges(n);
        }
        this.newGraph(node);
    }

    private class NodePopupMenuProvider implements PopupMenuProvider{
        public JPopupMenu getPopupMenu(Widget widget, Point localLocation) {
            Object o = ((ObjectScene)widget.getScene()).findObject(widget);
            if(o != null && o instanceof Node){
                JPopupMenu menu = ((Node)o).getContextMenu();
                if(o instanceof NodeNode){
                    addMoveAction(menu, (NodeNode)o);
                    addMinimizeAction(menu, (NodeNode)o);
                    return menu;
                } else if(o instanceof EdgeNode){
                    addMoveAction(menu, (EdgeNode)o);
                }
                return menu;
            }
            return null;
        }
    }

    private void addMinimizeAction(JPopupMenu menu, NodeNode o){
        JMenuItem item = new JMenuItem();
        Widget w = findWidget(o);
        if(w!=null && w instanceof NodeWidgetInterface){
            final NodeWidgetInterface nw = (NodeWidgetInterface) w;
            if(nw.minimized()){
                item.setAction(new AbstractAction(bundle.getString("restore")) {
                    public void actionPerformed(ActionEvent e) {
                        nw.restore();
                    }
                });
            } else {
                item.setAction(new AbstractAction(bundle.getString("minimize")) {
                    public void actionPerformed(ActionEvent e) {
                        nw.minimize();
                    }
                });
            }
            menu.add(item, 0);
        }
    }

    private void addMoveAction(JPopupMenu menu, NodeNode o){
        JMenu go = new JMenu(bundle.getString("goto"));
        JMenu before = new JMenu(bundle.getString("predecessors"));
        JMenu after = new JMenu(bundle.getString("successors"));
        List<AbstractGraphNode> pre = GraphUtils.getPredecessors(o.getNode().getAbstractNode());
        List<AbstractGraphNode> suc = GraphUtils.getSuccessors(o.getNode().getAbstractNode());
        for(AbstractGraphNode n : pre){
            final NodeNode node = findNodeNode(n);
            if(node!=null){
                JMenuItem i = new JMenuItem(new AbstractAction(node.getDisplayName()) {
                    public void actionPerformed(ActionEvent e) {
                        scrollToWidget(findWidget(node));
                    }
                });
                before.add(i);
            }
        }
        if(pre.isEmpty()){
            JMenuItem i = new JMenuItem(bundle.getString("nopredecessors"));
            i.setEnabled(false);
            before.add(i);
        }
        for(AbstractGraphNode n : suc){
            final NodeNode node = findNodeNode(n);
            if(node!=null){
                JMenuItem i = new JMenuItem(new AbstractAction(node.getDisplayName()) {
                    public void actionPerformed(ActionEvent e) {
                        scrollToWidget(findWidget(node));
                    }
                });
                after.add(i);
            }
        }
        if(suc.isEmpty()){
            JMenuItem i = new JMenuItem(bundle.getString("nosuccessors"));
            i.setEnabled(false);
            after.add(i);
        }
        go.add(before);
        go.add(after);
        Icon icon = right;
        if(icon!=null){
            go.setIcon(icon);
        }
        menu.add(go, 0);
    }

    private void addMoveAction(JPopupMenu menu, EdgeNode o){
        JMenu go = new JMenu(bundle.getString("goto"));
        final NodeNode source = this.getEdgeSource(o);
        final NodeNode destination = this.getEdgeTarget(o);
        JMenuItem before = new JMenuItem(new AbstractAction(moveString(bundle.getString("source"), source)) {
            public void actionPerformed(ActionEvent e) {
                scrollToWidget(findWidget(source));
            }
        });
        JMenuItem after = new JMenuItem(new AbstractAction(moveString(bundle.getString("destination"), destination)) {
            public void actionPerformed(ActionEvent e) {
                scrollToWidget(findWidget(destination));
            }
        });
        go.add(before);
        go.add(after);
        Icon icon = right;
        if(icon!=null){
            go.setIcon(icon);
        }
        menu.add(go, 0);
    }

    private void scrollToWidget(final Widget widget){
        if(widget==null) return;
        JViewport view = scrollPane.getViewport();
        Widget w;
        if(widget instanceof ConnectionWidget){
            w = ((ConnectionWidget)widget).getTargetAnchor().getRelatedWidget();
        } else {
            w = widget;
        }
        if(w==null) return;
        view.setViewPosition(new Point(this.convertSceneToView(w.getPreferredLocation()).x-view.getExtentSize().width/2+(int)(w.getPreferredBounds().getWidth()/2), this.convertSceneToView(w.getPreferredLocation()).y-view.getExtentSize().height/2+(int)(w.getPreferredBounds().getHeight()/2)));
    }

    private NodeNode findNodeNode(AbstractGraphNode n) {
        for(NodeNode node : this.getNodes()){
            if(node.getNode().getAbstractNode()==n) return node;
        }
        return null;
    }

    private String moveString(String message, Node node){
        StringBuilder b = new StringBuilder();
        b.append(message);
        b.append(" ");
        b.append(node.getDisplayName());
        return b.toString();
    }

    private class MoveProviderImpl implements MoveProvider{

        public void movementStarted(Widget widget) {
        }

        public void movementFinished(Widget widget) {

        }

        public Point getOriginalLocation(Widget widget) {
            return widget.getLocation();
        }

        public void setNewLocation(Widget widget, Point location) {
            GraphScene s = (GraphScene) widget.getScene();
            int x = location.x-widget.getLocation().x;
            int y = location.y-widget.getLocation().y;
            widget.setPreferredLocation(location);
            movedWidgets.add(widget);
        }

    }
}
