/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.scenes;

import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.gpdv.services.GraphLayouter;
import org.gpdv.core.api.layoutgraph.LayoutGraphEdge;
import org.gpdv.core.api.layoutgraph.LayoutGraph;
import org.gpdv.core.api.layoutgraph.LayoutGraphNode;
import org.gpdv.core.api.layoutgraph.LayoutGraphSlot;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * @author odi
 */
public class TestLayouter implements GraphLayouter{
    
    private final String NAME = "TESTLAYOUTER";
    private final String DISPLAYNAME = "TEST Force-based layouter";
    private final String DESCRIPTION = "TEST Force-based layouter included in GPDV bundle";

    private final int ITERATIONS = 10000;
    private final double DAMPING = 0.01;
    private final double MAXMOVEMENT = 5;

    public synchronized boolean layoutGraph(LayoutGraph graph) {
        if(graph==null) return false;
        List<LayoutGraphNode> nodes = graph.getLayoutNodes();
        List<LayoutGraphEdge> edges = graph.getLayoutEdges();
        List<LayoutGraphNode> roots = getRootNodes(graph);
        List<LayoutGraphNode> ends = getEndNodes(graph, roots);
        Map<LayoutGraphNode, NodeData> nodeMap = new HashMap<LayoutGraphNode, NodeData>();
        int between = nodes.size()-roots.size()-ends.size();
        fillNodeMap(nodes, nodeMap, Math.max(roots.size(), ends.size()), between);
        setFixedNodes(roots, nodeMap, 0);
        setFixedNodes(ends, nodeMap, between);
        int iterations = 0;
        while(iterations < ITERATIONS){
            calculateSpringForce(edges, nodeMap);
            calculateCoulombForce(nodes, nodeMap);
            applyForces(nodes, nodeMap, Math.max(roots.size()*2, ends.size()*2), between);
            iterations++;
        }
        applyNewCoordinates(nodes, nodeMap);
        setEdgeControlPoints(edges);
        return true;
    }

    public Node getRepresentation() {
        Node n = new AbstractNode(Children.LEAF);
        n.setName(NAME);
        n.setDisplayName(DISPLAYNAME);
        n.setShortDescription(DESCRIPTION);
        return n;
    }

    private List<LayoutGraphNode> getRootNodes(LayoutGraph graph){
        List<LayoutGraphNode> nodes = new ArrayList<LayoutGraphNode>();
        for(LayoutGraphNode n : graph.getLayoutNodes()){
            boolean root = true;
            for(LayoutGraphSlot slot : n.getIncomingLayoutSlots()){
                if(!slot.getLayoutEdges().isEmpty()){
                    root = false;
                }
            }
            if(root) nodes.add(n);
        }
        return nodes;
    }

    private List<LayoutGraphNode> getEndNodes(LayoutGraph graph, List<LayoutGraphNode> roots){
        List<LayoutGraphNode> nodes = new ArrayList<LayoutGraphNode>();
        for(LayoutGraphNode n : graph.getLayoutNodes()){
            boolean end = true;
            for(LayoutGraphSlot slot : n.getOutgoingLayoutSlots()){
                if(!slot.getLayoutEdges().isEmpty()){
                    end = false;
                }
            }
            if(end&&!roots.contains(n)) nodes.add(n);
        }
        return nodes;
    }

    private void setSlots(LayoutGraphNode node) {
        Point p = node.getPoint();
        int i = 0;
        int count = node.getIncomingLayoutSlots().size();
        for(LayoutGraphSlot s : node.getIncomingLayoutSlots()){
            s.setPoint(new Point((int)(p.x+(node.getWidth()/count*i)), (int)(p.y-s.getHeight())));
//            s.setPoint(new Point(node.getWidth()/count*i, -s.getHeight()));
            i++;
        }
        i = 0;
        count = node.getOutgoingLayoutSlots().size();
        for(LayoutGraphSlot s : node.getOutgoingLayoutSlots()){
            s.setPoint(new Point((p.x+(node.getWidth()/count*i)), (p.y+node.getHeight())));
//            s.setPoint(new Point(node.getWidth()/count*i, node.getHeight()));
            i++;
        }
    }

    private void setFixedNodes(List<LayoutGraphNode> nodes, Map<LayoutGraphNode, NodeData> map, int y) {
        for(int i = 0; i < nodes.size(); i++){
//            nodes.get(i).setPoint(new Point(i+i,y));
            NodeData d = map.get(nodes.get(i));
            d.x = i+i; d.y = y;
            d.fixed = true;
        }
    }

    private void fillNodeMap(List<LayoutGraphNode> nodes, Map<LayoutGraphNode, NodeData> nodeMap, int x, int y) {
        for(LayoutGraphNode n : nodes){
            nodeMap.put(n, new NodeData(x/2,y/2));
        }
    }

    private void calculateSpringForce(List<LayoutGraphEdge> edges, Map<LayoutGraphNode, NodeData> nodeMap) {
        for(LayoutGraphEdge e : edges){
            NodeData source = nodeMap.get(e.getSourceLayoutSlot().getLayoutNode());
            NodeData dest = nodeMap.get(e.getDestinationLayoutSlot().getLayoutNode());
            double velX = dest.x - source.x;
            double velY = dest.y - source.y;
            double length = (velX * velX + velY * velY);
            if(length==0) length = 0.001;
            double dx = DAMPING * velX * length;
	    double dy = DAMPING * velY * length;
            dest.dx -= dx;
            dest.dy -= dy;
            source.dx += dx;
            source.dy += dy;
        }
    }

    private void calculateCoulombForce(List<LayoutGraphNode> nodes, Map<LayoutGraphNode, NodeData> nodeMap) {
        for(int i = 0; i < nodes.size(); i++){
            double dx = 0;
            double dy = 0;
            NodeData d1 = nodeMap.get(nodes.get(i));
            for(int j = 0; j < nodes.size(); j++){
                if(j!=i){
                    NodeData d2 = nodeMap.get(nodes.get(j));
                    double velX = d1.x - d2.x;
                    double velY = d1.y - d2.y;
                    double length = velX * velX + velY * velY;
                    if(length!=0){
                        dx += velX / (length);
                        dy += velY / (length);
                    }
                }
            }
            double dLength = dx * dx + dy * dy;
            if(dLength > 0){
                dLength = Math.sqrt(dLength) / 2;
                d1.dx += dx / dLength;
                d1.dy += dy / dLength;
            }
        }
    }

    private void applyForces(List<LayoutGraphNode> nodes, Map<LayoutGraphNode, NodeData> nodeMap, int rightx, int downy) {
        for(LayoutGraphNode n : nodes){
            NodeData d = nodeMap.get(n);
            if(!d.fixed){
                d.x += Math.max(-MAXMOVEMENT, Math.min(MAXMOVEMENT, d.dx));
		d.y += Math.max(-MAXMOVEMENT, Math.min(MAXMOVEMENT, d.dy));
                if(d.x<0) d.x = 0;
                if(d.y<1) d.y = 1;
                if(d.x>rightx) d.x = rightx;
                if(d.y>downy-1) d.y = downy-1;
            }
            d.dx /= 5;
            d.dy /= 5;
        }
    }

    private void applyNewCoordinates(List<LayoutGraphNode> nodes, Map<LayoutGraphNode, NodeData> nodeMap) {
        double scaleFactor = calculateScaleFactor(nodes);
        for(LayoutGraphNode n : nodes){
            Point p = n.getPoint();
            NodeData d = nodeMap.get(n);
            Point newPoint = new Point((int)(d.x*scaleFactor)+((int)(Math.random()*1000)), (int)(d.y*scaleFactor)+((int)(Math.random()*1000)));
            n.setPoint(newPoint);
            setSlots(n);
        }
    }

    private double calculateScaleFactor(List<LayoutGraphNode> nodes) {
        double d = 1;
        for(LayoutGraphNode n : nodes){
            double nodeSize = Math.max(n.getHeight(), n.getWidth());
            double slotSize = 0;
            for(LayoutGraphSlot s : n.getIncomingLayoutSlots()){
                if(Math.max(s.getHeight(), s.getWidth())>slotSize) slotSize = Math.max(s.getHeight(), s.getWidth());
            }
            if(nodeSize+slotSize>d) d = nodeSize+slotSize;
        }
        return d;
    }

    private void setEdgeControlPoints(List<LayoutGraphEdge> edges) {
        for(LayoutGraphEdge edge : edges){
            if(edge.getSourceLayoutSlot().getLayoutNode()==edge.getDestinationLayoutSlot().getLayoutNode()){
                Point p = edge.getSourceLayoutSlot().getPoint();
                for(Point x : edge.getControlPoints()){
                    edge.removeControlPoint(x);
                }
                edge.addControlPoint(p);
                edge.addControlPoint(new Point(p.x, p.y+10));
                edge.addControlPoint(new Point(p.x-20, p.y+10));
                p = edge.getDestinationLayoutSlot().getPoint();
                edge.addControlPoint(new Point(p.x-20, p.y-10));
                edge.addControlPoint(new Point(p.x, p.y-10));
                edge.addControlPoint(p);
            } else {
                Point p = edge.getSourceLayoutSlot().getPoint();
                for(Point x : edge.getControlPoints()){
                    edge.removeControlPoint(x);
                }
                edge.addControlPoint(p);
                edge.addControlPoint(new Point(p.x + 0, p.y + 20));
                edge.addControlPoint(new Point(p.x + 20, p.y + 20));
                edge.addControlPoint(new Point(p.x + 20, p.y + 40));
                edge.addControlPoint(new Point(p.x + -20, p.y + 40));
                edge.addControlPoint(new Point(p.x + -20, p.y + 80));
                edge.addControlPoint(new Point(p.x + 40, p.y + 120));
            }
        }
    }

    public boolean isSlotAware() {
        return true;
    }

    private class NodeData{
        double x;
        double y;
        double dx;
        double dy;
        boolean fixed;
        public NodeData(int x, int y){
            this.x = x;
            this.y = y;
            dx = 0; dy = 0;
        }
    }

    @Override
    public String toString() {
        return this.DISPLAYNAME;
    }
}
