/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.views;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import org.gpdv.services.GraphBuilder;
import org.gpdv.services.GraphDesigner;
import org.gpdv.services.GraphLayouter;
import org.gpdv.services.validators.AbstractGraphValidator;
import org.gpdv.services.validators.DesignGraphValidator;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.DesignGraphFactory;
import org.gpdv.core.api.GraphView;
import org.gpdv.core.api.views.GraphExplorerTopComponent;
import org.gpdv.core.api.views.VisualizationTopComponent;
import org.gpdv.core.api.VisualizationRequest;
import org.gpdv.core.api.Visualizer;
import org.gpdv.core.implementation.nodes.EdgeNode;
import org.gpdv.core.implementation.nodes.GraphChangeEvent;
import org.gpdv.core.implementation.nodes.GraphChangeListener;
import org.gpdv.core.implementation.nodes.GraphNode;
import org.gpdv.core.implementation.nodes.NodeNode;
import org.gpdv.core.implementation.nodes.SlotNode;
import org.gpdv.core.implementation.scenes.GraphPinSceneImpl;
import org.gpdv.core.implementation.scenes.GraphSceneImpl;
import org.gpdv.core.implementation.scenes.GraphSceneInterface;
import org.gpdv.services.GraphRouter;
import org.netbeans.api.visual.action.SelectProvider;
import org.netbeans.api.visual.graph.GraphPinScene;
import org.netbeans.api.visual.graph.GraphScene;
import org.netbeans.api.visual.graph.layout.GraphLayout;
import org.netbeans.api.visual.graph.layout.GraphLayoutFactory;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.layout.SceneLayout;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.widget.Widget;
import org.netbeans.spi.navigator.NavigatorLookupHint;
import org.openide.actions.CopyAction;
import org.openide.actions.CutAction;
import org.openide.actions.DeleteAction;
import org.openide.actions.PasteAction;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.WeakListeners;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author odi
 */
public final class VisualizationManagementPanel extends GraphView implements ExplorerManager.Provider, Lookup.Provider, /*NavigatorLookupHint,*/ GraphChangeListener{

    private List<AbstractGraphValidator> availableAbstractGraphValidators;
    private List<DesignGraphValidator> availableDesignGraphValidators;
    private List<GraphRouter> availableRouters;
    private List<GraphDesigner> availableGraphDesigners;
    private List<GraphLayouter> availableGraphLayouters;
    private GraphBuilder nodeGraphBuilder;
    private GraphDesigner designer;
    private GraphLayouter layouter;
    private List<AbstractGraphValidator> agValidators;
    private List<DesignGraphValidator> dgValidators;
    private GraphNode graph;
    private GraphSceneInterface scene;
    private JToolBar toolBar;

    private NavigatorLookupHint hint;
    private InstanceContent content;

    private boolean processing;
    private boolean changesDuringProcessing;
    private boolean graphBuilding;
    private boolean changesDuringGraphBuilding;

    private ExplorerManager manager; 
    private ExplorerManager parentManager;
    private Lookup lookup;
    JComponent sceneComponent;
    JComponent satView;
    ManagerListener mListener;
    PropertyChangeListener wmListener;
    UnknownManagerListener umListener;
    boolean underTC;
    boolean designing;
    boolean defaultLayout;
    private VisualizationRequest request;
    private GraphRouter router;
    private boolean initiated;

    private boolean sceneInitiated;
    private boolean newDesigner;
    private boolean newLayouter;
    private boolean newNodes;


    public VisualizationManagementPanel(VisualizationRequest request){
        this(request, new ExplorerManager());
    }

    public VisualizationManagementPanel(VisualizationRequest request, ExplorerManager topManager){
        hint = new NavigatorLookupHint() {
            public String getContentType() {
                return "gpdvgraph";
            }
        };
        content = new InstanceContent();
        content.add(hint);
        content.add(this);
        Lookup lkp = new AbstractLookup(content);
        processing = false;
        changesDuringProcessing = false;
        graphBuilding = false;
        changesDuringGraphBuilding = false;
        this.request = request;
        this.manager = topManager;
        initActions();
        sceneInitiated = false;
        newDesigner = false;
        newLayouter = false;
        newNodes = false;
        underTC = false;
        designing = false;
        initiated = false;
        this.router = request.getRouter();
        if(request.getRouters()!=null&&!request.getRouters().isEmpty()){
            this.availableRouters = request.getRouters();
            if(this.router==null&&!this.availableRouters.isEmpty()){
                this.router = this.availableRouters.get(0);
            }
        } else {
            this.availableRouters = new ArrayList<GraphRouter>(Visualizer.Utils.getRouters());
        }
        if(this.router==Visualizer.Utils.getDefaultRouter()){
            this.router = null;
        }
        if(request.getDesigners()!=null&&!request.getDesigners().isEmpty()){
            // list of designers is given
            this.availableGraphDesigners = request.getDesigners();
        } else {
            // no list given, use all designers in system
            Collection<GraphDesigner> designers = Visualizer.Utils.getGraphDesignersForMimeType("universal");
            this.availableGraphDesigners = new ArrayList<GraphDesigner>(designers);
        }
        if(request.getPreferredDesigner()!=null){
            // preferred designer given
            designer = request.getPreferredDesigner();
            if(!this.availableGraphDesigners.contains(designer)){
                this.availableGraphDesigners.add(designer);
            }
        } else if(!availableGraphDesigners.isEmpty()){
            // no preferred designer, use first of possible designers
            designer = availableGraphDesigners.get(0);
        } else {
            // nothing given, use first designer in system
            designer = Lookup.getDefault().lookup(GraphDesigner.class);
        }
        if(request.getPreferredLayouter()!=null){
            // preferred layouter given
            if(request.getPreferredLayouter()!= Visualizer.Utils.getDefaultLayouter()) layouter = request.getPreferredLayouter();
        }   // nothing given, leave layouter null -> use default layouter
        if(request.getLayouter()!=null&&!request.getLayouter().isEmpty()){
            // list of layouters is given
            this.availableGraphLayouters = request.getLayouter();
        } else {
            // no list given, use all layouters in system
            this.availableGraphLayouters = new ArrayList<GraphLayouter>(Visualizer.Utils.getGraphLayouters());
        }
        if(request.getPreferredAbstractGraphValidators()!=null){
            this.agValidators = new ArrayList<AbstractGraphValidator>(request.getPreferredAbstractGraphValidators());
        } else {
            this.agValidators = new ArrayList<AbstractGraphValidator>();
        }
        if(request.getAbstractGraphValidators()!=null&&!request.getAbstractGraphValidators().isEmpty()){
            this.availableAbstractGraphValidators = new ArrayList<AbstractGraphValidator>(request.getAbstractGraphValidators());
        } else {
            this.availableAbstractGraphValidators = new ArrayList<AbstractGraphValidator>(Visualizer.Utils.getAbstractGraphValidators());
        }
        if(request.getPreferredDesignGraphValidators()!=null){
            this.dgValidators = new ArrayList<DesignGraphValidator>(request.getPreferredDesignGraphValidators());
        } else {
            this.dgValidators = new ArrayList<DesignGraphValidator>();
        }
        if(request.getDesignGraphValidators()!=null&&!request.getDesignGraphValidators().isEmpty()){
            this.availableDesignGraphValidators = new ArrayList<DesignGraphValidator>(request.getDesignGraphValidators());
        } else {
            this.availableDesignGraphValidators = new ArrayList<DesignGraphValidator>(Visualizer.Utils.getDesignGraphValidators());
        }
        this.lookup = new ProxyLookup(ExplorerUtils.createLookup(this.manager, getActionMap()), lkp);//, Lookups.singleton(this));
    }

    public List<GraphRouter> getAvailableRouters() {
        return new ArrayList<GraphRouter>(availableRouters);
    }

    public List<GraphDesigner> getAvailableGraphDesigners() {
        return new ArrayList<GraphDesigner>(availableGraphDesigners);
    }

    public List<GraphLayouter> getAvailableGraphLayouters() {
        return new ArrayList<GraphLayouter>(availableGraphLayouters);
    }

    public GraphDesigner getSelectedDesigner(){
        return designer;
    }

    public GraphLayouter getSelectedLayouter(){
        if(layouter == null) return Visualizer.Utils.getDefaultLayouter();
        return layouter;
    }



    @Override
    public void addNotify() {
        super.addNotify();
        if(initiated){
            return;
        }
        initiated = true;
        mListener = new ManagerListener();
        wmListener = WeakListeners.propertyChange(mListener, manager);
        if(SwingUtilities.getAncestorOfClass(VisualizationTopComponent.class, this)!=null || SwingUtilities.getAncestorOfClass(GraphExplorerTopComponent.class, this)!=null){
            // Panel is used in VisualizationTopComponent or GraphExplorerTopComponent
            // listening for parentManager changes are performed by listening on manager
            underTC = true;
            createGraph();
        } else {
            // Panel is NOT used in VisualizationTopComponent
            if(request.getAbstractGraph()!=null||request.getDesignGraph()!=null){
                // panel should be used for graph!!!
                // GENERATE NODEREPRESENTATION
                // SET MANAGER
                // CALL NEWGRAPH
                createGraph();
            } else {
                // panel is used with empty request
                // it should visualize the parents explorermanager contents
                this.parentManager = ExplorerManager.find(this);
                if(parentManager!=null){
                    // listen for changes from unknown parentManager
                    // umListener listens for any changes in parentManager and fires event which triggers newTreeGraph()
                    //  newTreeGraph() gets data from parent manager, converts is and sets own manager which fires an event to trigger newGraph()
                    umListener = new UnknownManagerListener();
                    parentManager.addPropertyChangeListener(WeakListeners.propertyChange(umListener, parentManager));
                    ExplorerUtils.activateActions(manager, true);
                    this.manager.addPropertyChangeListener(wmListener);
                    newNodes = true;
                    newDesigner = true;
                    newLayouter = true;
                    scheduleNewTreeGraph();
                }
            }

        }
    }

    private void createGraph(){
        Node n = null;
        if(request.getDesignGraph()!=null){
            n = Visualizer.Utils.getNodeRepresentation(request.getDesignGraph());
        } else if(request.getAbstractGraph()!=null){
            n = Visualizer.Utils.getNodeRepresentation(request.getAbstractGraph());
        }
        if(n!=null){
            manager.setRootContext(n);
        } else {
            manager.setRootContext(Node.EMPTY);
        }
        this.manager.addPropertyChangeListener(wmListener);
        newDesigner = true;
        newNodes = true;
        newLayouter = true;
        scheduleNewGraph();
    }

    public ExplorerManager getExplorerManager() {
        return manager;
    }

    private void newTreeGraph() {
        // get data from parent manager
        // create graph based on data
        // get node representation for graph
        // set manager to created node representation
        changesDuringGraphBuilding = false;
//        scene = new GraphSceneImpl(request, new SelectProviderImpl(manager), this);
//        sceneInitiated = false;
        if(nodeGraphBuilder==null){
            Collection<GraphBuilder> graphBuilders = Visualizer.Utils.getGraphBuildersForMimeType("netbeans/node");
            Iterator<GraphBuilder> i = graphBuilders.iterator();
            while(nodeGraphBuilder==null&&i.hasNext()){
                GraphBuilder b = i.next();
                Node node = parentManager.getRootContext();
                if(b.accepts(node)) nodeGraphBuilder = b;
            }
        }
        if(nodeGraphBuilder==null||parentManager==null){
            graphBuildingError();
            errorOccured();
            return;
        }
        SwingWorker<AbstractGraph, Void> worker = new SwingWorker<AbstractGraph, Void>() {
            @Override
            protected AbstractGraph doInBackground() throws Exception {
                return nodeGraphBuilder.buildGraph(parentManager.getRootContext());
            }
            @Override
            protected void done() {
                try {
                    afterGraphBuilding(get());
                } catch (Exception ex) {
                    graphBuildingError();
                    errorOccured();
                    return;
                }
            }
        };
        worker.execute();
    }

    private void afterGraphBuilding(AbstractGraph gr){
        if(gr==null){
            graphBuildingError();
            errorOccured();
            return;
        }
        if(changesDuringGraphBuilding){
            scheduleNewTreeGraph();
            return;
        }
        DesignGraph g = DesignGraphFactory.getFactory().convertAbstractToDesignGraph(gr);
        GraphNode n = (GraphNode) Visualizer.Utils.getNodeRepresentation(g);
        manager.setRootContext(n);
        scheduleNewGraph();
    }

    public Lookup getLookup() {
        return this.lookup;
    }

    private void initActions() {
        CutAction cut = SystemAction.get(CutAction.class);
        getActionMap().put(cut.getActionMapKey(), ExplorerUtils.actionCut(manager));

        CopyAction copy = SystemAction.get(CopyAction.class);
        getActionMap().put(copy.getActionMapKey(), ExplorerUtils.actionCopy(manager));

        PasteAction paste = SystemAction.get(PasteAction.class);
        getActionMap().put(paste.getActionMapKey(), ExplorerUtils.actionPaste(manager));

        DeleteAction delete = SystemAction.get(DeleteAction.class);
        getActionMap().put(delete.getActionMapKey(), ExplorerUtils.actionDelete(manager, true));
    }

    private void newGraph() {
        content.remove(hint);
        content.remove(this);
        changesDuringProcessing = false;
//        scene = new GraphSceneImpl(request, new SelectProviderImpl(manager), this); // init scene
//        sceneInitiated = false;
        if((manager.getRootContext()==null || manager.getRootContext().equals(Node.EMPTY))){
            errorOccured();
            return;
        }    // no context set, possible in combination with standalone topcomponent
        if(graph!=null) graph.removeGraphChangeListener(this);
        graph = (GraphNode) manager.getRootContext();
        // start listening for graphchanges
        graph.addGraphChangeListener(this);
        for(Node n : graph.getChildren().getNodes()){
            for(Node m : n.getChildren().getNodes()){
                for(Node b : m.getChildren().getNodes()){
                    System.out.print("");
                }
            }
        }
        System.out.println("");
//        if(scene!=null) scene.clear();
        if(layouter!=null&&layouter.isSlotAware()&&request.allowPins()){
            // Layouter NOT NULL AND
            // Layouter IS SLOTAWARE
            // AND Pins are allowed
            if(scene==null||scene instanceof GraphSceneImpl){
//                System.out.println("NEW ___GRAPH_PIN_SCENE___ CREATED");
                scene = new GraphPinSceneImpl(request, new SelectProviderImpl(manager), this);
                sceneInitiated = false;
            }
        } else {
            if(scene==null||scene instanceof GraphPinSceneImpl){
//                System.out.println("NEW ___GRAPH SCENE___ CREATED");
                scene = new GraphSceneImpl(request, new SelectProviderImpl(manager), this);
                sceneInitiated = false;
            }
        }
        if(designer==null){
            errorOccured();
            return;
        }
        scene.setRouter(router);
        designing = true;   // DO NOT REACT TO CHANGES OF SLOTS, graph still updates itself
        final GraphDesigner d = designer;
        final DesignGraph dg = graph.getGraph();
        final List<AbstractGraphValidator> agv = new ArrayList<AbstractGraphValidator>(agValidators);
        final List<DesignGraphValidator> dgv = new ArrayList<DesignGraphValidator>(dgValidators);
        if(newDesigner||newNodes){
//            System.out.println("NEW DESIGNER || NEW NODES ---> run designer");
            SwingWorker<Boolean, Void> worker = new SwingWorker<Boolean, Void>() {
                @Override
                protected Boolean doInBackground() throws Exception {
                    boolean succ = d.designGraph(dg);
                    for(AbstractGraphValidator v : agv){
                        v.validate(dg);
                    }
                    for(DesignGraphValidator v : dgv){
                        v.validate(dg);
                    }
                    return succ;
                }
                @Override
                protected void done() {
                    try {
                        afterDesigning(get());
                    } catch (Exception ex) {
//                        Exceptions.printStackTrace(ex);
                        errorOccured();
                        return;
                    }
                }
            };
            worker.execute();
        } else {
//            System.out.println("SKIP DESIGNER");
            afterDesigning(true);
        }
    }

    private void afterDesigning(boolean success){
        designing = false;
        if(!success){
            errorOccured();
            return;
        }
        if(changesDuringGraphBuilding){
            scheduleNewTreeGraph();
            return;
        }
        if(changesDuringProcessing){
            scheduleNewGraph();
            return;
        }
        if(newLayouter||newNodes||scene instanceof GraphPinSceneImpl){ // new layouter -> layouter has to run, changes in nodes -> run layouter if nodes were added and need to be placed
//            System.out.println("NEW LAYOUTER || NEW NODES || GraphPinScene and new design ---> run layouter");
            newLayouter = false;
            newNodes = false;
            if(layouter!=null){
                final GraphLayouter l = layouter;
                SwingWorker<Boolean, Void> worker = new SwingWorker<Boolean, Void>() {
                    @Override
                    protected Boolean doInBackground() throws Exception {
//                        Thread.sleep(10000);
                        return l.layoutGraph(graph.getGraph());
                    }
                    @Override
                    protected void done() {
                        try {
                            afterLayouting(get());
                        } catch (Exception ex) {
//                            Exceptions.printStackTrace(ex);
                            errorOccured();
                            return;
                        }
                    }
                };
                worker.execute();
            } else {
                afterLayouting(false);
            }
        } else {
          //no layouting needed, just draw nodes
//          System.out.println("SKIP LAYOUTING");
          if(layouter==null){
              afterLayouting(false);
          } else {
              afterLayouting(true);
          }
        }
    }

    private void afterLayouting(boolean success){

        if(success){
            if(newDesigner){
//                System.out.println("TOTALLY NEW GRAPH");
                scene.totallyNewGraph(graph);
                newDesigner = false;
            } else {
//                System.out.println("NEW GRAPH");
                scene.newGraph(graph);
            }
        } else {
            // Layouter NULL or Layouter NOT NULL AND FAILED
            // FALLBACK if layouter was not null and slotaware (-> GraphPinScene) but layouting failed
            // does NOT change anything if layouter was already null or not slotaware
            if(scene instanceof GraphPinScene){
                scene = new GraphSceneImpl(request, new SelectProviderImpl(manager), this);
                sceneInitiated = false;
            }
            if(newDesigner){
//                System.out.println("TOTALLY NEW GRAPH");
                scene.totallyNewGraph(graph);
                newDesigner = false;
            } else {
//                System.out.println("NEW GRAPH");
                scene.newGraph(graph);
            }
            GraphScene gsc = scene.getGraphScene();
            if(gsc!=null){
                GraphLayout gl = GraphLayoutFactory.createHierarchicalGraphLayout(gsc, false);
                SceneLayout scl = LayoutFactory.createSceneGraphLayout(gsc, gl);
                // set router for scenelayout
                this.router = null;
                scene.setFreeRouter(true);
                scl.invokeLayoutImmediately();
            }

        }
        if(changesDuringGraphBuilding){
            scheduleNewTreeGraph();
            return;
        }
        if(changesDuringProcessing){
            scheduleNewGraph();
            return;
        }
        setScene();
        resetProcessing();
        if(manager.getSelectedNodes().length>0){
            changeSelectedNodes(new HashSet<Object>(Arrays.asList(manager.getSelectedNodes())));
        } else {
            try {
                manager.setSelectedNodes(new Node[]{manager.getRootContext()});
            } catch (PropertyVetoException ex) {
//                Exceptions.printStackTrace(ex);
            }
        }
    }

    private void errorOccured(){
        setScene();
        resetProcessing();
    }

    private void setScene(){
        if(!sceneInitiated){
            sceneComponent = scene.getObjectScene().createView();
            sceneInitiated = true;
        }
        this.setViewportView(sceneComponent);
    }

    private void resetProcessing(){
        processing = false;
        graphBuilding = false;
        content.add(hint);
        content.add(this);
    }

    public String getContentType() {
        return "gpdvgraph";
    }

    @Override
    public JToolBar getToolBar() {
        if(!request.allowToolbar()) return null;
        if(this.toolBar==null) this.toolBar = new MyToolBar(request, this);
        return toolBar;
    }

    @Override
    public void enforceLayout() {
        if(!request.allowRelayouting()) return;
        // redo layout -> run layouter!
        if(!processing){
            this.enforceLayout(false);
        }
        // if processing is true when this call occurs then:
        //      builder, designer, layouter runs during newGraph procedure
        //      layouter runs during set layouter procedure
        // in both cases the set layouter does the layouting and the widgets will be moved to there assigned position
        // AFTER this call anyway, so no action needed
    }

    private void enforceLayout(boolean necessary) {
        // redo layout -> run layouter!
        if(request.allowRelayouting()||necessary){
            if(layouter!=null){
                graphBuilding = true;
                processing = true;
                final GraphLayouter l = layouter;
                final Component c = this;
                newLayouter = false;
                SwingWorker<Boolean, Void> worker = new SwingWorker<Boolean, Void>() {
                    @Override
                    protected Boolean doInBackground() throws Exception {
                        return l.layoutGraph(graph.getGraph());
                    }
                    @Override
                    protected void done() {
                        try {
                            boolean b = get();
                            if(changesDuringGraphBuilding){
                                scheduleNewTreeGraph();
                                return;
                            } else if(changesDuringProcessing){
                                scheduleNewGraph();
                                return;
                            }
                            graphBuilding = false;
                            processing = false;
                            if(b){
                                scene.enforceLayout(true);
                                // set router for non-scenelayout
                                scene.setFreeRouter(false);
                                scene.getObjectScene().revalidate();
                                SwingUtilities.getRoot(c).validate();
                                SwingUtilities.getRoot(c).repaint();
                            } else {
                                resetNullLayouter();
                            }
                        } catch (InterruptedException ex) {
//                            Exceptions.printStackTrace(ex);
                        } catch (ExecutionException ex) {
//                            Exceptions.printStackTrace(ex);
                        }
                    }
                };
                worker.execute();
            } else {
                // no layouter set
                resetNullLayouter();
            }
        }
    }
    
    private void resetNullLayouter(){
        GraphScene gsc = scene.getGraphScene();
        if(gsc!=null){
            // GraphScene
            GraphLayout gl = GraphLayoutFactory.createHierarchicalGraphLayout(gsc, false);
            SceneLayout scl = LayoutFactory.createSceneGraphLayout(gsc, gl);
            // set router for scenelayout
            scene.setFreeRouter(true);
            this.router = null;
            scl.invokeLayoutImmediately();
        } else {
            // no layouter set & GraphPinScene -> newGraph needed
            scheduleNewGraph();
        }
    }


    @Override
    public void setGraphLayouter(GraphLayouter layouter) {
        // run layouter and call enforceLayout
        // NO newGraph NEEDED SINCE SIZES AND ELEMENTS (SLOTS) DO NOT CHANGE
        // UNLESS (!!!) new layouter requires different kind of scene!!!
        if(request.allowDifferentLayouter()&&(this.availableGraphLayouters.contains(layouter)||layouter==null)){
            if(this.layouter!=null&&layouter!=null&&this.layouter==layouter) return;
            if(this.layouter==null&&(layouter==null||layouter==Visualizer.Utils.getDefaultLayouter())) return;
            boolean newSceneRequired = false;
            if(this.layouter==null&&layouter!=null&&layouter.isSlotAware()&&request.allowPins()){
                // GraphScene (no layouter) -> GraphPinScene (slotaware layouter)
                // not necessary if no slots should be used at all
                newSceneRequired = true;
            } else if(this.layouter!=null&&!this.layouter.isSlotAware()&&layouter!=null&&layouter.isSlotAware()&&request.allowPins()){
                // GraphScene (Not SlotAware Layouter) -> GraphPinScene (SlotAware GraphLayouter)
                // not necessary if no slots should be used at all
                newSceneRequired = true;
            } else if(this.layouter!=null&&this.layouter.isSlotAware()&&request.allowPins()&&(layouter==null||!layouter.isSlotAware())){
                // GraphPinScene (SlotAware Layouter) -> GraphScene (No or Not SlotAware Layouter)
                newSceneRequired = true;
            } else if(layouter!=null&&layouter.isSlotAware()&&request.allowPins()&&(scene instanceof GraphSceneImpl)){
                // Old layouter was slotaware and pins are allowed, but hat problems during layouting and fallback to graphscene was used
                newSceneRequired = true;
            }
            if(layouter!=Visualizer.Utils.getDefaultLayouter()){
                this.layouter = layouter;
            } else {
                this.layouter = null;
            }
            newLayouter = true;
            if(processing){
                // this call happens during:
                //      graphBuilder run
                //      designer run
                //      layouter run
                // after layouter returned the first possible point of execution is after the scene is fully rendered and layouted
                // (since everything after layouter return happens in awt thread), and then processing would not be true anymore...
                // so if processing is true then this call happens BEFORE changesDuringProcessing is checked at least once again
                changesDuringProcessing = true;
                return;
            }
            if(newSceneRequired){
                // call newGraph to create new scene
                scheduleNewGraph();
            } else {
                this.enforceLayout(true);
            }
        }
    }

    @Override
    public void setGraphDesigner(GraphDesigner designer) {
        if(request.allowDifferentDesigner()&&this.availableGraphDesigners.contains(designer)){
            if(this.designer == designer) return;
            this.designer = designer;
            if(!processing){
                newDesigner = true;
//                System.out.println("NEW DESIGNER");
                scheduleNewGraph();
            } else {
                changesDuringProcessing = true;
            }
        }
    }

    @Override
    public void setAbstractGraphValidators(List<AbstractGraphValidator> validators) {
        if(request.allowDifferentValidator()){
            this.agValidators.clear();
            for(AbstractGraphValidator v : validators){
                if(this.availableAbstractGraphValidators.contains(v)){
                    this.agValidators.add(v);
                }
            }
            if(processing){
                changesDuringProcessing = true;
            } else {
                scheduleNewGraph();
            }
        }
    }

    @Override
    public void setDesignGraphValidators(List<DesignGraphValidator> validators) {
        if(request.allowDifferentValidator()){
            this.dgValidators.clear();
            for(DesignGraphValidator v : validators){
                if(this.availableDesignGraphValidators.contains(v)){
                    this.dgValidators.add(v);
                }
            }
            if(processing){
                changesDuringProcessing = true;
            } else {
                scheduleNewGraph();
            }
        }
    }

    @Override
    public JComponent getSatelliteView() {
        if(processing) return null;
        if(scene==null||!sceneInitiated) return new JPanel();
        return scene.getObjectScene().createSatelliteView();
    }

    public void graphChangePerformed(GraphChangeEvent e) {
        // DO NOT REACT TO CHANGES HAPPENING DURING DESIGNING!!!
        // -> DESIGNER CAN CHANGE SLOTS -> SLOTLISTCHILDREN UPDATES IT'S CHILDREN
        // newGraph() WOULD BE CALLED FOR EVERY CHANGED SLOT!
        if(SwingUtilities.isEventDispatchThread()){
            newNodes = true;
            if(processing){
                changesDuringProcessing = true;
            } else {
                scheduleNewGraph();
            }
        }
    }

    private void scheduleNewGraph(){
        processing = true;
//        manager.removePropertyChangeListener(wmListener);
//        try {
//            manager.setSelectedNodes(new Node[]{manager.getRootContext()});
//        } catch (PropertyVetoException ex) {
//            Exceptions.printStackTrace(ex);
//        }
        Runnable r = new Runnable() {
            public void run() {
                newGraph();
            }
        };
        SwingUtilities.invokeLater(r);
    }

    @Override
    public void resetLayout() {
        if(request.allowResetLayout()){
            if(processing){
                return;
            }
            if(layouter==null){
                // no layouter
                // cant just reset to old position, call this.enforceLayout
                // will run scene layouter again -> calculate old positions again
                this.enforceLayout(true);
            } else {
                scene.enforceLayout(false);
//                scene.setFreeRouter(false);
                scene.getObjectScene().revalidate();
                SwingUtilities.getRoot(this).validate();
                SwingUtilities.getRoot(this).repaint();
            }
        }
    }

    @Override
    public void setRouter(GraphRouter router) {
            if(!request.allowDifferentRouter()) return;
            if((router==null)&&this.router==null||this.router==router) return;
            // a layouter is used -> no scenelayout
            if(router!=Visualizer.Utils.getDefaultRouter()){
                this.router = router;
            } else {
                this.router = null;
            }
            scene.setRouter(this.router);
            if(layouter!=null||this.router!=null){
                // set router for non-scenelayout
                scene.setFreeRouter(false);
                scene.getObjectScene().revalidate();
                SwingUtilities.getRoot(this).validate();
                SwingUtilities.getRoot(this).repaint();
            }else {
                resetNullLayouter();
            }
    }

    private void scheduleNewTreeGraph() {
        processing = true;
        graphBuilding = true;
        Runnable r = new Runnable() {
            public void run() {
                newTreeGraph();
            }
        };
        SwingUtilities.invokeLater(r);
    }

    private void graphBuildingError() {
        manager.setRootContext(Node.EMPTY);
        scene = new GraphSceneImpl(request, new SelectProviderImpl(manager), this);
        sceneInitiated = false;
    }
    
    private class SelectProviderImpl implements SelectProvider{

        private ExplorerManager manager;

        public SelectProviderImpl(ExplorerManager manager){
            this.manager = manager;
        }

        public boolean isAimingAllowed(Widget widget, Point localLocation, boolean invertSelection) {
            return false;
        }

        public boolean isSelectionAllowed(Widget widget, Point localLocation, boolean invertSelection) {
            return true;
        }

        public synchronized void select(Widget widget, Point localLocation, boolean invertSelection) {
            // invertSelection: pressed CTRL
            ObjectScene scene = (ObjectScene) widget.getScene();
            Object o = scene.findObject(widget);
            List<Node> nodes1 = Arrays.asList(manager.getSelectedNodes());
            List<Node> nodes = new ArrayList<Node>();
            for(Node n : nodes1){
                nodes.add(n);
            }
            if(o == null){
                try {
                    manager.setSelectedNodes(new Node[]{manager.getRootContext()}/*new Node[0]*/);
                } catch (PropertyVetoException ex) {
//                    Exceptions.printStackTrace(ex);
                }
            }
            if(o instanceof Node){
                try {
                    if(invertSelection){
                        // add new selected node to currently selected nodes
                        if(nodes.contains((Node)o)){
                            nodes.remove((Node)o);
                        } else {
                            nodes.add((Node)o);
                        }
                        manager.setSelectedNodes(nodes.toArray(new Node[0]));
                    } else {
                        // set selected nodes to only the new selected node
                        manager.setSelectedNodes(new Node[]{(Node) o});
                    }
                } catch (PropertyVetoException ex) {
//                    Exceptions.printStackTrace(ex);
                }
            }
        }
    }

    public synchronized void changeSelectedNodes(Set<Object> objects) {
        // invertSelection already processed by selectProvider -> false
        if(objects==null||objects.size()==0){
            if(scene!=null){
                if(scene.getObjectScene()!=null&&!processing){
                    scene.getObjectScene().setSelectedObjects(Collections.emptySet());
                }
            }
            return;
        }
        Set<Object> objectsToSelect = new HashSet<Object>();
        for(Object o : objects){
            if((o instanceof NodeNode || o instanceof EdgeNode) && scene.getObjectScene().findWidget(o)!=null) objectsToSelect.add(o);
            if((o instanceof SlotNode && scene instanceof GraphPinScene) && scene.getObjectScene().findWidget(o)!=null) objectsToSelect.add(o);
        }
        Object first = null;
        for(Object o : objectsToSelect){
            if(o instanceof NodeNode || (o instanceof SlotNode && scene instanceof GraphPinScene)){
                first = o;
                break;
            }
        }
        scene.getObjectScene().userSelectionSuggested(objectsToSelect, false);
        Widget w = scene.getObjectScene().findWidget(first);
        if(w!=null){
            Rectangle r = this.getViewport().getViewRect();
            double zoom = scene.getObjectScene().getZoomFactor();
            ObjectScene sc = scene.getObjectScene();
            Rectangle wi = new Rectangle(sc.convertSceneToView(w.getPreferredLocation()), new Dimension((int)(w.getPreferredSize().width*zoom), (int)(w.getPreferredSize().height*zoom)));
            if(!r.intersects(wi)){
                scene.focusSelectedObject();
            }
        }
        scene.getObjectScene().revalidate();
        SwingUtilities.getRoot(this).validate();
        SwingUtilities.getRoot(this).repaint();
    }

    private class UnknownManagerListener implements PropertyChangeListener{

        public void propertyChange(PropertyChangeEvent evt) {
            if(evt.getPropertyName().equals(ExplorerManager.PROP_ROOT_CONTEXT)||evt.getPropertyName().equals(ExplorerManager.PROP_NODE_CHANGE)){
                if(graphBuilding){
                    changesDuringGraphBuilding = true;
                } else {
                    scheduleNewTreeGraph();
                }
            }
            if(evt.getPropertyName().equals(ExplorerManager.PROP_SELECTED_NODES)){
                // map parentManager nodes -> manager nodes
                    // based on node names
                // get Nodes representing the selected nodes in parentmanager
                // set selected nodes in manager
                if(graphBuilding) return;
                Node root = getExplorerManager().getRootContext();
                Node n1 = root.getChildren().getNodeAt(0);
                if(n1!=null){
                    Children c = n1.getChildren();
                    if(parentManager.getSelectedNodes().length>0){
                        Node selected = parentManager.getSelectedNodes()[0];
                        Node n = c.findChild(selected.getDisplayName());
                        if(n==null){
                            // error
                        } else {
                            try {
                                manager.setSelectedNodes(new Node[]{n});
                            } catch (PropertyVetoException ex) {
//                                Exceptions.printStackTrace(ex);
                            }
                        }
                    }
                }
            }
        }

    }

    private class ManagerListener implements PropertyChangeListener{

        public void propertyChange(PropertyChangeEvent evt) {
            if(evt.getPropertyName().equals(ExplorerManager.PROP_ROOT_CONTEXT)/*||evt.getPropertyName().equals(ExplorerManager.PROP_NODE_CHANGE)*/) {
            // remove all listeners reacting to graph changes!  // not necessary ---> changes are processed by nodes and corresponding manager
                if(processing){
                    changesDuringProcessing = true;
                } else {
                    scheduleNewGraph();
                }
            } else {
               if(evt.getPropertyName().equals(ExplorerManager.PROP_SELECTED_NODES)){
                   if(!processing)changeSelectedNodes(new HashSet<Object>(Arrays.asList(manager.getSelectedNodes())));
               }
            }
        }
    }
}
