/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.widgets;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.implementation.nodes.NodeNode;
import org.netbeans.api.visual.border.BorderFactory;
import org.netbeans.api.visual.laf.LookFeel;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.model.ObjectState;
import org.netbeans.api.visual.widget.ImageWidget;
import org.netbeans.api.visual.widget.LabelWidget;
import org.netbeans.api.visual.widget.LabelWidget.Alignment;
import org.netbeans.api.visual.widget.LabelWidget.VerticalAlignment;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import org.netbeans.api.visual.widget.general.IconNodeWidget;

/**
 *
 * @author odi
 */
public class ImageNodeWidget extends Widget implements NodeWidgetInterface{

    private NodeNode node;
    private List<Widget> slots;
    private Design design;
    private boolean minimized;

    private ImageWidget imageWidget;
    private LabelWidget labelWidget;

    public ImageNodeWidget(NodeNode node, Scene scene) {
        super(scene);
        this.minimized = false;
        this.design = node.getNode().getDesign();
        LookFeel lookFeel = getScene().getLookFeel();
        setLayout(LayoutFactory.createVerticalFlowLayout(LayoutFactory.SerialAlignment.CENTER, -lookFeel.getMargin()+2));
        setLayout(LayoutFactory.createAbsoluteLayout());
        this.node = node;
        this.labelWidget = new LabelNodeWidget(node, scene);
        this.imageWidget = new ImageWidget(scene);
        slots = new ArrayList<Widget>();
        this.setToolTipText(node.getShortDescription());
        this.setPreferredLocation(node.getNode().getPoint());
        Image image = node.getNode().getDesign().getImage();
        int labelHeight = node.getNode().getDesign().getHeight()-image.getHeight(null);
        int labelWidth = node.getNode().getDesign().getWidth();
        labelWidget.setPreferredBounds(new Rectangle(0, 0, labelWidth, labelHeight));
        labelWidget.setPreferredSize(new Dimension(labelWidth, labelHeight));
        labelWidget.setPreferredLocation(new Point(0,image.getHeight(null)));
        imageWidget.setImage(node.getNode().getDesign().getImage());
        imageWidget.setPreferredLocation(new Point(node.getNode().getWidth()/2-image.getWidth(null)/2,0));
        this.setPreferredBounds(new Rectangle(0, 0, node.getNode().getWidth(), node.getNode().getHeight()));
        this.setPreferredSize(new Dimension(node.getNode().getWidth(), node.getNode().getHeight()));
        addChild(imageWidget);
        addChild(labelWidget);
    }

    @Override
    public void notifyStateChanged(ObjectState previousState, ObjectState state) {
        super.notifyStateChanged(previousState, state);
        for(Widget w : this.getChildren()){
            w.setState(state);
        }
        if(state.isSelected()){
//            labelWidget.setBackground(node.getNode().getDesign().getSelectedColor());
            this.setOpaque(minimized);
            this.bringToFront();
            for(Widget w : this.getSlots()) w.bringToFront();
            if(!minimized)this.setPreferredBounds(new Rectangle(-design.getSelectionSizeIncreaseHorizontal(),-design.getSelectionSizeInncreaseVertical(), node.getNode().getWidth()+2*design.getSelectionSizeIncreaseHorizontal(), node.getNode().getHeight()+2*design.getSelectionSizeInncreaseVertical()));
            if(!minimized){
                this.setBorder(BorderFactory.createRoundedBorder(10, 10, design.getIncreasedBackgroundColor(), Color.black));
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getSelectedColor(), design.getSelectedBorderColor()));
            }
//            this.setForeground(Color.WHITE);
//            this.setFont(this.getFont().deriveFont(Font.BOLD));
        } else if(state.isHovered()){
//            labelWidget.setBackground(node.getNode().getDesign().getHoveredColor());
            this.setOpaque(true);
            this.bringToFront();
            for(Widget w : this.getSlots()) w.bringToFront();
//            this.setBackground(Color.lightGray);
//            this.setPreferredBounds(new Rectangle(-2,-2, node.getNode().getWidth()+4, node.getNode().getHeight()+4));
//            this.setBorder(BorderFactory.createBevelBorder(true, Color.black));
            // TODO: ADD DESIGNINFORMATION ON IMAGE HOVER AND SELECTED BACKGROUND COLORS AND SIZE-INCREASE
            if(!minimized)this.setPreferredBounds(new Rectangle(-design.getSelectionSizeIncreaseHorizontal(),-design.getSelectionSizeInncreaseVertical(), node.getNode().getWidth()+2*design.getSelectionSizeIncreaseHorizontal(), node.getNode().getHeight()+2*design.getSelectionSizeInncreaseVertical()));
            if(!minimized){
                this.setBorder(BorderFactory.createRoundedBorder(10, 10, design.getIncreasedBackgroundColor(), Color.black));
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getHoveredColor(), design.getHoveredBorderColor()));
            }
//            this.setForeground(Color.YELLOW);
//            this.setFont(this.getFont().deriveFont(Font.ITALIC));
        } else {
//            if(this.labelWidget==null||this.node==null) return;
//            labelWidget.setBackground(node.getNode().getDesign().getDefaultColor());
            this.setOpaque(minimized);
            if(!minimized)this.setPreferredBounds(new Rectangle(0, 0, design.getWidth(), design.getHeight()));
            if(!minimized){
                this.setBorder(BorderFactory.createEmptyBorder());
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getDefaultColor(), design.getDefaultBorderColor()));
            }
//            this.setForeground(Color.BLACK);
//            this.setFont(this.getFont().deriveFont(Font.PLAIN));
        }
    }

    public void addSlotWidget(Widget w){
        slots.add(w);
    }

    public List<Widget> getSlots(){
        return slots;
    }

    public NodeNode getNode() {
        return node;
    }

    public void minimize() {
        if(minimized) return;
        for(Widget w : slots){
            ((SlotWidget)w).minimize(this.getPreferredLocation());
        }
        this.removeChild(imageWidget);
        this.removeChild(labelWidget);
        this.setPreferredBounds(new Rectangle(0,0,design.getMinimizedWidth(),design.getMinimizedHeight()));
        this.minimized = true;
        this.setOpaque(true);
        this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getDefaultColor(), design.getDefaultBorderColor()));
        this.revalidate();
    }

    public void restore() {
        if(!minimized) return;
        for(Widget w : slots){
            ((SlotWidget)w).restore();
        }
        this.addChild(imageWidget);
        this.addChild(labelWidget);
        this.setPreferredBounds(new Rectangle(0,0,design.getWidth(),design.getHeight()));
        this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidth(), design.getArcHeight(), design.getDefaultColor(), design.getDefaultBorderColor()));
        this.minimized = false;
        this.revalidate();
    }

    public boolean minimized(){
        return minimized;
    }
}
