/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.implementation.widgets;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.implementation.nodes.NodeNode;
import org.netbeans.api.visual.border.BorderFactory;
import org.netbeans.api.visual.model.ObjectState;
import org.netbeans.api.visual.widget.LabelWidget;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;

/**
 *
 * @author odi
 */
public class LabelNodeWidget extends LabelWidget implements NodeWidgetInterface{
    private NodeNode node;
    private List<Widget> slots;
    private Design design;
    private boolean minimized;

    public LabelNodeWidget(NodeNode node, Scene scene) {
        super(scene);
        this.minimized = false;
        this.design = node.getNode().getDesign();
        this.node = node;
        slots = new ArrayList<Widget>();
        this.setLabel(node.getDisplayName());
        this.setToolTipText(node.getShortDescription());
        this.setPreferredLocation(node.getNode().getPoint());
        this.setAlignment(Alignment.CENTER);
        this.setVerticalAlignment(VerticalAlignment.CENTER);
        //this.setBorder(BorderFactory.createBevelBorder(true, Color.black));
        this.setBorder(BorderFactory.createRoundedBorder(0, 0, design.getDefaultColor(), design.getDefaultBorderColor()));
        this.setForeground(node.getNode().getDesign().getDefaultFontColor());
        this.setPreferredBounds(new Rectangle(0, 0, node.getNode().getWidth(), node.getNode().getHeight()));
        this.setPreferredSize(new Dimension(node.getNode().getWidth(), node.getNode().getHeight()));
        this.setOpaque(true);
        this.setBackground(design.getDefaultColor());
    }

    @Override
    public void notifyStateChanged(ObjectState previousState, ObjectState state) {
        super.notifyStateChanged(previousState, state);
        if(state.isSelected()){
//            this.setBackground(node.getNode().getDesign().getSelectedColor());
            if(!minimized){
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidth(), design.getArcHeight(), design.getSelectedColor(), design.getSelectedBorderColor()));
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getSelectedColor(), design.getSelectedBorderColor()));
            }
            this.setForeground(design.getSelectedFontColor());
            this.bringToFront();
            for(Widget w : this.getSlots()) w.bringToFront();
//            this.setForeground(Color.WHITE);
//            this.setFont(this.getFont().deriveFont(Font.BOLD));
        } else if(state.isHovered()){
            if(!minimized){
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidth(), design.getArcHeight(), design.getHoveredColor(), design.getHoveredBorderColor()));
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getHoveredColor(), design.getHoveredBorderColor()));
            }
            this.setForeground(node.getNode().getDesign().getHoveredFontColor());
            this.bringToFront();
            for(Widget w : this.getSlots()) w.bringToFront();
//            this.setForeground(Color.YELLOW);
//            this.setFont(this.getFont().deriveFont(Font.ITALIC));
        } else {
            if(!minimized){
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidth(), design.getArcHeight(), design.getDefaultColor(), design.getDefaultBorderColor()));
            } else {
                this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getDefaultColor(), design.getDefaultBorderColor()));
            }
            this.setForeground(node.getNode().getDesign().getDefaultFontColor());
//            this.setForeground(Color.BLACK);
//            this.setFont(this.getFont().deriveFont(Font.PLAIN));
        }
    }

    public void addSlotWidget(Widget w){
        slots.add(w);
    }

    public List<Widget> getSlots(){
        return slots;
    }

    public NodeNode getNode() {
        return node;
    }

    public void minimize() {
        if(minimized) return;
        for(Widget w : slots){
            ((SlotWidgetInterface)w).minimize(this.getPreferredLocation());
        }
        this.setPreferredBounds(new Rectangle(0,0,design.getMinimizedWidth(),design.getMinimizedHeight()));
        this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidthMinimized(), design.getArcHeightMinimized(), design.getDefaultColor(), design.getDefaultBorderColor()));
        this.setLabel("");
        this.minimized = true;
    }

    public void restore() {
        if(!minimized) return;
        for(Widget w : slots){
            ((SlotWidgetInterface)w).restore();
        }
        this.setPreferredBounds(new Rectangle(0,0,design.getWidth(),design.getHeight()));
        this.setBorder(BorderFactory.createRoundedBorder(design.getArcWidth(), design.getArcHeight(), design.getDefaultColor(), design.getDefaultBorderColor()));
        this.setLabel(node.getDisplayName());
        this.minimized = false;
    }

    public boolean minimized(){
        return minimized;
    }
}
