/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.abstractgraph;

import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import java.util.Map;
import java.util.Set;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author odi
 */
public class AbstractEdgeTest {

    private AbstractGraph g1, g2;
    private AbstractGraphNode n1, n2, n3, n4;
    private AbstractGraphEdge e;
    private String name;
    private long id;


    public AbstractEdgeTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
        id = 7L;
        name = "TestEdge";
        g1 = new AbstractGraph(1L, "TestGraph1", null);
        g2 = new AbstractGraph(2L, "TestGraph2", null);
        n1 = new AbstractGraphNode(3L, "TestNode1", g1, null);
        n2 = new AbstractGraphNode(4L, "TestNode2", g1, null);
        n3 = new AbstractGraphNode(5L, "TestNode3", g2, null);
        n4 = new AbstractGraphNode(6L, "TestNode4", g2, null);
        e = new AbstractGraphEdge(id, name, g1, n1, n2, null);
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of getGraph method, of class AbstractGraphEdge.
     */
    @Test
    public void testGetGraph() {
        System.out.println("getGraph");
        assertEquals(e.getGraph(), g1);
    }

    /**
     * Test of getID method, of class AbstractGraphEdge.
     */
    @Test
    public void testGetID() {
        System.out.println("getID");
        assertEquals(id, e.getID());
    }

    /**
     * Test of getName method, of class AbstractGraphEdge.
     */
    @Test
    public void testGetName() {
        System.out.println("getName");
        assertEquals(name, e.getName());
    }

    /**
     * Test of getSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testGetSource() {
        System.out.println("getSource");
        assertEquals(n1, e.getSource());
    }

    /**
     * Test of getDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testGetDestination() {
        System.out.println("getDestination");
        assertEquals(n2, e.getDestination());
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSource() {
        System.out.println("setSource");
        assertTrue(e.setSource(n2));
        assertEquals(e.getSource(), n2);
        assertFalse(n1.getOutgoingEdges().contains(e));
        assertTrue(n2.getOutgoingEdges().contains(e));
        assertTrue(e.setSource(n1));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceRemoved() {
        System.out.println("setSourceRemoved");
        g1.removeEdge(e);
        assertTrue(empty(n1)&&empty(n2));
        assertTrue(e.setSource(n2));
        assertTrue(empty(n1)&&empty(n2));
        g1.removeNode(n1);
        assertFalse(e.setSource(n1));
        assertTrue(empty(n1)&&empty(n2));
        g1.addNode(n1);
        assertTrue(empty(n1)&&empty(n2));
        assertTrue(e.setSource(n1));
        assertTrue(empty(n1)&&empty(n2));
        g1.addEdge(e);
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
    }
    
    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceToSource() {
        System.out.println("setSourceToSource");
        assertFalse(e.setSource(n1));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setSource(n1));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceToRemovedNode() {
        System.out.println("setSourceToRemovedNode");
        g1.removeNode(n2);
        assertFalse(e.setSource(n2));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceNull() {
        System.out.println("setSourceNull");
        assertFalse(e.setSource(null));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setSource(null));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceToOtherGraphNode() {
        System.out.println("setSourceToOtherGraphNode");
        assertFalse(e.setSource(n3));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        assertFalse(n3.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setSource(n3));
        assertTrue(empty(n1)&&empty(n2)&&empty(n3));
    }

    /**
     * Test of setSource method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetSourceToOtherRemovedGraphNode() {
        System.out.println("setSourceToOtherRemovedGraphNode");
        g2.removeNode(n3);
        assertFalse(e.setSource(n3));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        assertFalse(n3.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setSource(n3));
        assertTrue(empty(n1)&&empty(n2)&&empty(n3));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestination() {
        System.out.println("setDestination");
        assertTrue(e.setDestination(n1));
        assertEquals(e.getDestination(), n1);
        assertTrue(n1.getIncomingEdges().contains(e));
        assertFalse(n2.getIncomingEdges().contains(e));
        assertTrue(e.setDestination(n2));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationRemoved() {
        System.out.println("setDestinationRemoved");
        g1.removeEdge(e);
        assertTrue(empty(n1)&&empty(n2));
        assertTrue(e.setDestination(n1));
        assertTrue(empty(n1)&&empty(n2));
        g1.removeNode(n2);
        assertFalse(e.setDestination(n2));
        assertTrue(empty(n1)&&empty(n2));
        g1.addNode(n2);
        assertTrue(empty(n1)&&empty(n2));
        assertTrue(e.setDestination(n2));
        assertTrue(empty(n1)&&empty(n2));
        g1.addEdge(e);
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationToDestination() {
        System.out.println("setDestinationToDestination");
        assertFalse(e.setDestination(n2));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setDestination(n2));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationToRemovedNode() {
        System.out.println("setDestinationToRemovedNode");
        g1.removeNode(n1);
        assertFalse(e.setDestination(n1));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationNull() {
        System.out.println("setDestinationNull");
        assertFalse(e.setDestination(null));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setDestination(null));
        assertTrue(empty(n1)&&empty(n2));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationToOtherGraphNode() {
        System.out.println("setDestinationToOtherGraphNode");
        assertFalse(e.setDestination(n3));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        assertFalse(n3.getIncomingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setDestination(n3));
        assertTrue(empty(n1)&&empty(n2)&&empty(n3));
    }

    /**
     * Test of setDestination method, of class AbstractGraphEdge.
     */
    @Test
    public void testSetDestinationToOtherRemovedGraphNode() {
        System.out.println("setDestinationToOtherRemovedGraphNode");
        g2.removeNode(n3);
        assertFalse(e.setDestination(n3));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertFalse(n1.getIncomingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertFalse(n2.getOutgoingEdges().contains(e));
        assertFalse(n3.getIncomingEdges().contains(e));
        g1.removeEdge(e);
        assertFalse(e.setDestination(n3));
        assertTrue(empty(n1)&&empty(n2)&&empty(n3));
    }

    private boolean empty(AbstractGraphNode node){
        return node.getOutgoingEdges().isEmpty()&&node.getIncomingEdges().isEmpty();
    }
}