/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.core.api.abstractgraph;

import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.abstractgraph.AbstractGraphChangeEvent;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphChangeListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.openide.util.Exceptions;

/**
 *
 * @author odi
 */
public class AbstractGraphTest implements AbstractGraphChangeListener{

    private AbstractGraph graph;
    private List<EventObject> events;

    public AbstractGraphTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
        this.graph = new AbstractGraph(1L, "TestGraph", null);
        this.graph.addAbstractGraphChangeListener(this);
        events = new ArrayList<EventObject>();
    }

    @After
    public void tearDown() {
        this.graph.removeAbstractGraphChangeListener(this);
    }

    /**
     * Test of getName method, of class AbstractGraph.
     */
    @Test
    public void testGetName() {
        System.out.println("getName");
        AbstractGraph instance = graph;
        String expResult = "TestGraph";
        String result = instance.getName();
        assertEquals(expResult, result);
    }

    /**
     * Test of getID method, of class AbstractGraph.
     */
    @Test
    public void testGetID() {
        System.out.println("getID");
        AbstractGraph instance = graph;
        long expResult = 1L;
        long result = instance.getID();
        assertEquals(expResult, result);
    }

    /**
     * Test of getNodes method, of class AbstractGraph.
     */
    @Test
    public void testGetNodes() {
        System.out.println("getNodes");
        assertTrue(graph.getNodes().isEmpty());
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode1", graph, null);
        AbstractGraphNode n2 = new AbstractGraphNode(3L, "TestNode2", graph, null);
        assertEquals(events.size(), 2);
        assertTrue(graph.getNodes().contains(n1));
        assertTrue(graph.getNodes().contains(n2));
        graph.removeNode(n2);
        assertEquals(events.size(), 3);
        assertTrue(graph.getNodes().contains(n1));
        assertFalse(graph.getNodes().contains(n2));
    }

    /**
     * Test of getEdges method, of class AbstractGraph.
     */
    @Test
    public void testGetEdges() {
        System.out.println("getEdges");
        assertTrue(graph.getEdges().isEmpty());
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode1", graph, null);
        AbstractGraphNode n2 = new AbstractGraphNode(3L, "TestNode2", graph, null);
        AbstractGraphEdge e = new AbstractGraphEdge(4L, "TestEdge", graph, n1, n2, null);
        assertEquals(events.size(), 3);
        assertTrue(graph.getEdges().contains(e));
        graph.removeEdge(e);
        assertEquals(events.size(), 4);
        assertTrue(graph.getEdges().isEmpty());
    }

    /**
     * Test of addNode method, of class AbstractGraph.
     */
    @Test
    public void testAddNode() {
        System.out.println("addNode");
        AbstractGraphNode node = new AbstractGraphNode(2L, "TestNode", graph, null);
        // node was inserted during creation
        assertTrue(graph.getNodes().contains(node));
        assertFalse(graph.getNodes().isEmpty());
        // inserting again is not possible
        assertFalse(graph.addNode(node));
        // inserting a previously removed node is possible
        graph.removeNode(node);
        assertTrue(graph.addNode(node));
        assertFalse(graph.getNodes().isEmpty());
        // inserting a node belonging to another graph is not possible
        assertEquals(events.size(), 3);
        AbstractGraph g = new AbstractGraph(1L, "TestGraph2", null);
        AbstractGraphNode n = new AbstractGraphNode(2L, "TestNode2", g, null);
        assertFalse(graph.addNode(n));
        g.removeNode(n);
        assertFalse(graph.addNode(n));
        // inserting null is not possible
        assertFalse(graph.addNode(null));
    }

    /**
     * Test of addEdge method, of class AbstractGraph.
     */
    @Test
    public void testAddEdge() {
        System.out.println("addEdge");
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode1", graph, null);
        AbstractGraphNode n2 = new AbstractGraphNode(3L, "TestNode2", graph, null);
        AbstractGraphEdge e = null;
        AbstractGraph g = new AbstractGraph(1L, "TestGraph2", null);
        AbstractGraphNode n3 = new AbstractGraphNode(4L, "TestNode3", g, null);
        AbstractGraphNode n4 = new AbstractGraphNode(5L, "TestNode4", g, null);
        AbstractGraphEdge e2;
        e = new AbstractGraphEdge(4L, "TestEdge", graph, n1, n2, null);
        // edge was inserted
        assertFalse(graph.getEdges().isEmpty());
        // edge can't be inserted again
        assertFalse(graph.addEdge(e));
        // null can't be inserted
        assertFalse(graph.addEdge(null));
        // edge belonging to other graph can't be inserted
        e2 = new AbstractGraphEdge(6L, "TestEdge2", g, n3, n4, null);
        assertFalse(graph.addEdge(e2));
        g.removeEdge(e2);
        assertFalse(graph.addEdge(e2));
        // edge with node not inserted can't be inserted
        graph.removeNode(n2);
        assertTrue(graph.getEdges().isEmpty());
        assertFalse(graph.addEdge(e));
        // removed edge can be re-inserted after all nodes are inserted
        graph.addNode(n2);
        assertTrue(graph.addEdge(e));
        assertEquals(events.size(), 7);
        // adding null is nor possible
        assertFalse(graph.addEdge(null));
    }

    /**
     * Test of removeNode method, of class AbstractGraph.
     */
    @Test
    public void testRemoveNode() {
        System.out.println("removeNode");
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode", graph, null);
        assertTrue(graph.getNodes().contains(n1));
        assertFalse(graph.getNodes().isEmpty());
        assertTrue(graph.removeNode(n1));
        assertTrue(graph.getNodes().isEmpty());
        assertFalse(graph.removeNode(n1));
        // removing null is not possible
        assertFalse(graph.removeNode(null));
        assertEquals(events.size(), 2);
    }

    /**
     * Test of removeNode method when edges are attached to the node, of class AbstractGraph.
     */
    @Test
    public void testRemoveNodeWithEdges() {
        System.out.println("removeNodeWithEdges");
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode", graph, null);
        AbstractGraphNode n2 = new AbstractGraphNode(3L, "TestNode2", graph, null);
        AbstractGraphEdge e = new AbstractGraphEdge(4L, "TestEdge", graph, n1, n2, null);
        // graph contains n1, n2, e
        assertTrue(graph.getNodes().contains(n1));
        assertTrue(graph.getNodes().contains(n2));
        assertTrue(n1.getOutgoingEdges().contains(e));
        assertTrue(n2.getIncomingEdges().contains(e));
        assertTrue(graph.getEdges().contains(e));
        assertFalse(graph.getNodes().isEmpty());
        assertFalse(graph.getEdges().isEmpty());
        // removing n1 is possible
        assertTrue(graph.removeNode(n1));
        // graph still contains n2, n1 was removed (with e)
        assertTrue(graph.getNodes().contains(n2));
        assertFalse(graph.getNodes().contains(n1));
        assertTrue(graph.getEdges().isEmpty());
        assertTrue(n2.getIncomingEdges().isEmpty());
        assertTrue(n1.getOutgoingEdges().isEmpty());
        // removing n1 is not possible
        assertFalse(graph.removeNode(n1));
        // removing null is not possible
        assertFalse(graph.removeNode(null));
        assertEquals(events.size(), 5);
    }

    /**
     * Test of removeEdge method, of class AbstractGraph.
     */
    @Test
    public void testRemoveEdge() {
        System.out.println("removeEdge");
        AbstractGraphNode n1 = new AbstractGraphNode(2L, "TestNode", graph, null);
        AbstractGraphNode n2 = new AbstractGraphNode(3L, "TestNode2", graph, null);
        AbstractGraphEdge e = new AbstractGraphEdge(4L, "TestEdge", graph, n1, n2, null);
        assertTrue(graph.getEdges().contains(e));
        assertTrue(graph.removeEdge(e));
        assertFalse(graph.removeEdge(e));
        assertFalse(graph.removeEdge(null));
        assertEquals(events.size(), 4);
    }

    public void abstractGraphChangePerformed(AbstractGraphChangeEvent e) {
        events.add(e);
//        AbstractGraphChangeEvent ev = (AbstractGraphChangeEvent)e;
//        System.out.print(ev.getChangeString() + " -> ");
//        if(ev.getChangedNode()!=null) System.out.print("Node: "+ev.getChangedNode().getName()+" ");
//        if(ev.getChangedEdge()!=null) System.out.print("Edge: "+ev.getChangedEdge().getName()+" ");
//        if(ev.getChangedProperties()!=null){
//            System.out.print("Key-Value-Pairs: ");
//            for(String key : ev.getChangedProperties().keySet()){
//                System.out.print(key+":"+ev.getChangedProperties().get(key)+" ");
//            }
//        }
//        System.out.println("");
    }

    /**
     * Test of addAbstractGraphChangeListener method, of class AbstractGraph.
     */
    @Test
    public void testAddAbstractGraphChangeListener() {
    }

    /**
     * Test of removeAbstractGraphChangeListener method, of class AbstractGraph.
     */
    @Test
    public void testRemoveAbstractGraphChangeListener() {
    }

    /**
     * Test of getProperties method, of class AbstractGraph.
     */
    @Test
    public void testGetProperties() {
    }

    /**
     * Test of addProperty method, of class AbstractGraph.
     */
    @Test
    public void testAddProperty() {
    }

    /**
     * Test of removeProperty method, of class AbstractGraph.
     */
    @Test
    public void testRemoveProperty() {
    }

    /**
     * Test of setProperties method, of class AbstractGraph.
     */
    @Test
    public void testSetProperties() {
    }

    /**
     * Test of getValue method, of class AbstractGraph.
     */
    @Test
    public void testGetValue() {
    }

    /**
     * Test of getKeys method, of class AbstractGraph.
     */
    @Test
    public void testGetKeys() {
    }

    /**
     * Test of abstractEdgeChangePerformed method, of class AbstractGraph.
     */
    @Test
    public void testAbstractEdgeChangePerformed() {
    }

    /**
     * Test of abstractNodeChangePerformed method, of class AbstractGraph.
     */
    @Test
    public void testAbstractNodeChangePerformed() {
    }
}