/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.imagepersondesigner;

import java.awt.Color;
import java.awt.Image;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.ResourceBundle;
import java.util.Set;
import org.gpdv.core.api.designgraph.Design;
import org.gpdv.core.api.designgraph.DesignGraphEdge;
import org.gpdv.core.api.designgraph.DesignGraph;
import org.gpdv.core.api.designgraph.DesignGraphNode;
import org.gpdv.core.api.designgraph.DesignGraphSlot;
import org.gpdv.services.GraphDesigner;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphDesigner.class)
public class ImagePersonDesignerImpl implements GraphDesigner{

    private static ResourceBundle bundle = NbBundle.getBundle(ImagePersonDesignerImpl.class);

    public Node getRepresentation() {
        AbstractNode n = new AbstractNode(Children.LEAF);
        n.setName(bundle.getString("name"));
        n.setDisplayName(bundle.getString("displayname"));
        n.setShortDescription(bundle.getString("description"));
        return n;
    }

    public boolean designGraph(DesignGraph graph) {
        for(DesignGraphNode n : graph.getDesignNodes()){
            this.designNode(n);
        }
        for(DesignGraphEdge e : graph.getDesignEdges()){
            this.designEdge(e);
        }
        return true;
    }

    public boolean designEdge(DesignGraphEdge edge) {
        edge.setDesign(getEdgeDesign());
        return true;
    }

    public boolean designNode(DesignGraphNode node) {
        Image image = node.getAbstractNode().getLookup().lookup(Image.class);
        if(image==null){
            image = ImageUtilities.loadImage(bundle.getString("anonymous"));
        }
        int width = 0;
        int height = 0;
        if(image!=null){
            width = image.getWidth(null);
            height = image.getHeight(null);
        }
        Color[] l = new Color[10];
        String s = node.getAbstractNode().getValue("Sex");
        if(s!=null){
            if(s.equals("male")){
                l[Design.BACKGROUND_POSITION] = new Color(10, 217, 215);
                l[Design.INCREASED_BACKGROUND_POSITION] = new Color(10, 217, 215);
            } else if(s.equals("female")){
                l[Design.BACKGROUND_POSITION] = Color.PINK;
                l[Design.INCREASED_BACKGROUND_POSITION] = Color.PINK;
            }
            l[Design.FONT_POSITION] = Color.WHITE;
        }
        int[] v = Design.getDefaultValues();
        v[Design.SELECTIONSIZEINCREASE_HORIZONTAL_POSITION] = 20;
        v[Design.SELECTIONSIZEINCREASE_VERTICAL_POSITION] = 10;
        v[Design.ARCWIDTH_POSITION] = 10;
        v[Design.ARCHEIGHT_POSITION] = 5;
        Design d = new Design(l, v, height+30, Math.max(width, node.getAbstractNode().getName().length()*8), image);
        node.setDesign(d);
        return true;
    }

    public boolean designSlot(DesignGraphSlot slot) {
        return true;
    }

    private Design getEdgeDesign(){
        Color[] c = new Color[10];
        c[Design.BACKGROUND_POSITION] = Color.BLACK;
        c[Design.BACKGROUND_HOVERED_POSITION] = Color.RED;
        c[Design.BACKGROUND_SELECTED_POSITION] = Color.GREEN;
        return new Design(c,0,0);
    }

    public Collection<String> getMimeTypes() {
        Set<String> s = new HashSet<String>();
        s.add(bundle.getString("image"));
        return Collections.unmodifiableCollection(s);
    }

}
