/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.services.modulegraphbuilder;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;
import org.gpdv.services.GraphBuilder;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.gpdv.core.api.AbstractGraphFactory;
import org.netbeans.api.project.Project;
import org.openide.modules.Dependency;
import org.openide.modules.ModuleInfo;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author odi
 */
@ServiceProvider(service=GraphBuilder.class)

public class ModuleGraphBuilderImpl implements GraphBuilder{

    private static ResourceBundle bundle = NbBundle.getBundle(ModuleGraphBuilderImpl.class);

    private AbstractGraphFactory factory;
    private AbstractGraph graph;
    private Set<Class> acceptedClasses;
    private int c;

    public Node getRepresentation() {
        Node n = new org.openide.nodes.AbstractNode(Children.LEAF);
        n.setName(bundle.getString("NAME"));
        n.setDisplayName(bundle.getString("DISPLAYNAME"));
        n.setShortDescription(bundle.getString("DESCRIPTION"));
        return n;
    }

    public boolean acceptsNull() {
        return false;
    }

    public AbstractGraph buildGraph(Object object) {
        if(object==null){
            return null;
        } if(object instanceof ModuleInfo){
            return buildGraphModuleInfo((ModuleInfo) object);
        } else {
            return null;
        }
    }

    private AbstractGraph buildGraphModuleInfo(ModuleInfo moduleInfo) {
        c = 0;
        if(factory==null){
            factory = AbstractGraphFactory.getFactory();
        }
        Collection<? extends ModuleInfo> modules = Lookup.getDefault().lookupAll(ModuleInfo.class);
//        for(ModuleInfo info : modules){
//            if(info.getCodeNameBase().contains("mime")) System.out.println(info.getCodeNameBase());
//        }
        Set<ModuleInfo> resolved = new HashSet<ModuleInfo>();
        Set<Dependency> resolvedUnloaded = new HashSet<Dependency>();
        AbstractGraph g = AbstractGraphFactory.getFactory().createAbstractGraph(moduleInfo.getDisplayName(), null);
        addModule(moduleInfo, g, modules, resolved, resolvedUnloaded);
        g.addProperty(bundle.getString("root"), moduleInfo.getCodeNameBase());
        return g;
    }

    private AbstractGraphNode addModule(ModuleInfo info, AbstractGraph graph, Collection<? extends ModuleInfo> modules, Set<ModuleInfo> resolved, Set<Dependency> resolvedUnloaded){
        AbstractGraphNode node = factory.createAbstractNode(info.getDisplayName(), graph, info);
        node.addProperty(bundle.getString("DISPLAYNAMEMODULE"), info.getDisplayName());
        node.addProperty(bundle.getString("CODENAME"), info.getCodeName());
        node.addProperty(bundle.getString("CODENAMEBASE"), info.getCodeNameBase());
        node.addProperty(bundle.getString("CODENAMEBASERELEASE"), Integer.toString(info.getCodeNameRelease()));
        node.addProperty(bundle.getString("BUILDVERSION"), info.getBuildVersion());
        node.addProperty(bundle.getString("SPECIFICATIONVERSION"), info.getSpecificationVersion().toString());
        node.addProperty(bundle.getString("IMPLEMENTATIONVERSION"), info.getImplementationVersion());
        resolved.add(info);
        Set<Dependency> deps = info.getDependencies();
        for (Dependency d : deps) {
            ModuleInfo i = getInfo(d, modules);
            if (i == null) {
                if (!resolvedUnloaded.contains(d)) {
                    resolvedUnloaded.add(d);
                    AbstractGraphNode n = factory.createAbstractNode(d.getName(), graph, d);
                    n.addProperty("Version", d.getVersion());
                    n.addProperty(bundle.getString("RESOLVED"), bundle.getString("NOTRESOLVED"));
                    factory.createAbstractEdge(bundle.getString("edge") + c, graph, node, n, null);
                    c++;
                }
            } else {
                AbstractGraphNode n;
                if (!resolved.contains(i)) {
                    n = addModule(i, graph, modules, resolved, resolvedUnloaded);
                } else {
                    n = find(i, graph);
                }
                if (n != null) {
                    factory.createAbstractEdge(bundle.getString("edge") + c, graph, node, n, null);
                    c++;
                }
            }
        }
        return node;
    }

    private ModuleInfo getInfo(Dependency d, Collection<? extends ModuleInfo> modules){
        for(ModuleInfo i : modules){
            if(i.getCodeNameBase().equals(d.getName())) return i;
        }
        return null;
    }

    public boolean accepts(Object object) {
        if(object==null) return acceptsNull();
        return (Project.class.isAssignableFrom(object.getClass()) || ModuleInfo.class.isAssignableFrom(object.getClass()));
    }

    public Collection<String> getMimeTypes() {
        List<String> l = new ArrayList<String>();
        l.add(new String(bundle.getString("MODULE")));
        return Collections.unmodifiableCollection(l);
    }

    private AbstractGraphNode find(ModuleInfo i, AbstractGraph graph) {
        for(AbstractGraphNode n : graph.getNodes()){
            if(n.getName().equals(i.getDisplayName())) return n;
        }
        return null;
    }

}
