/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.gpdv.Services.NodeGraphBuilder;

import org.gpdv.services.nodegraphbuilder.NodeGraphBuilderImpl;
import java.util.ArrayList;
import java.util.List;
import org.gpdv.services.GraphBuilder;
import org.gpdv.core.api.abstractgraph.AbstractGraphEdge;
import org.gpdv.core.api.abstractgraph.AbstractGraph;
import org.gpdv.core.api.abstractgraph.AbstractGraphNode;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.openide.nodes.Children;
import static org.junit.Assert.*;
import org.openide.nodes.Node;

/**
 *
 * @author odi
 */
public class NodeGraphBuilderImplTest {

    private static GraphBuilder graphBuilder;

    public NodeGraphBuilderImplTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        graphBuilder = new NodeGraphBuilderImpl();
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    /**
     * Test of buildGraph method, of class NodeGraphBuilderImpl.
     */
    @Test
    public void testBuildGraph() {
        org.openide.nodes.AbstractNode node = new org.openide.nodes.AbstractNode(new MyChildren(10));
        node.setName("root");
        node.setDisplayName("root");
//        printTree(node);
        AbstractGraph g = graphBuilder.buildGraph(node);
        System.out.println("Graph created");
        System.out.println("Number of nodes: "+g.getNodes().size());
        System.out.println("Number of edges: "+g.getEdges().size());
        assertEquals(g.getNodes().size(), 21);
        assertEquals(g.getEdges().size(), 20);
        System.out.println("Graph structure:");
        AbstractGraphNode n = g.getNodes().get(0);
        printNode(n);
    }

    private void printNode(AbstractGraphNode n){
        for(AbstractGraphEdge e : n.getOutgoingEdges()){
            System.out.println(e.getSource().getName() + " -> " + e.getDestination().getName());
            printNode(e.getDestination());
        }
    }

    private void printTree(org.openide.nodes.Node node){
        System.out.println(node.getDisplayName());
        for(org.openide.nodes.Node n : node.getChildren().getNodes()){
            printTree(n);
        }
    }

    /**
     * Test of acceptsNull method, of class NodeGraphBuilderImpl.
     */
    @Test
    public void testAcceptsNull() {
        assertFalse(graphBuilder.acceptsNull());
    }

    /**
     * Test of accepts method, of class NodeGraphBuilderImpl.
     */
    @Test
    public void testAccepts() {
        assertTrue(graphBuilder.accepts(new org.openide.nodes.AbstractNode(Children.LEAF)));
        assertFalse(graphBuilder.accepts(new Object()));
        assertFalse(graphBuilder.accepts(null));
    }

    /**
     * Test of getMimeTypes method, of class NodeGraphBuilderImpl.
     */
    @Test
    public void testGetMimeTypes() {
        assertTrue(graphBuilder.getMimeTypes().contains("netbeans/node"));
        assertEquals(graphBuilder.getMimeTypes().size(), 1);
    }

    private class MyChildren extends Children.Keys<Integer>{

        private int c;

        public MyChildren(int c){
            super();
            this.c = c;
        }

        @Override
        protected void addNotify() {
            super.addNotify();
            List<Integer> ints = new ArrayList<Integer>();
            for(int i = 1; i <= c; i++){
                ints.add(i);
            }
            setKeys(ints);
        }



        @Override
        protected Node[] createNodes(Integer i) {
            org.openide.nodes.AbstractNode n = n = new org.openide.nodes.AbstractNode(new MyChildren2(i));
            n.setName("Node " + i);
            n.setDisplayName("Node " + i);
            return new Node[]{n};
        }

    }

    private class MyChildren2 extends Children.Keys<Integer>{

        private int c;

        public MyChildren2(int c){
            super();
            this.c = c;
        }

        @Override
        protected void addNotify() {
            super.addNotify();
            List<Integer> ints = new ArrayList<Integer>();
            for(int i = c; i <= c; i++){
                ints.add(i);
            }
            setKeys(ints);
        }



        @Override
        protected Node[] createNodes(Integer i) {
            org.openide.nodes.AbstractNode n = n = new org.openide.nodes.AbstractNode(Children.LEAF);
            n.setName("Node " + i + " "+i);
            n.setDisplayName("Node " + i + " " +i);
            return new Node[]{n};
        }

    }

}